@@ -1,14 +1,15 @@
 //
 // ActionHandler.cc for pekwm
-// Copyright (C) 2002-2006 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2008 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id: ActionHandler.cc 116 2006-09-11 14:18:55Z me@pekdon.net $
-//
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #include "ActionHandler.hh"
 
 #include "PWinObj.hh"
@@ -20,10 +21,10 @@
 #include "Config.hh"
 #include "CmdDialog.hh"
 #include "Workspaces.hh"
-#include "Viewport.hh"
 #include "WindowManager.hh"
 #include "Util.hh"
 #include "RegexString.hh"
+#include "WorkspaceIndicator.hh"
 
 #ifdef HARBOUR
 #include "Harbour.hh"
@@ -56,21 +57,20 @@ using std::auto_ptr;
 using std::string;
 using std::list;
 using std::find;
+using std::map;
 
 ActionHandler *ActionHandler::_instance = NULL;
 
 //! @brief ActionHandler constructor
-ActionHandler::ActionHandler(WindowManager *wm) :
-        _wm(wm)
+ActionHandler::ActionHandler(void)
 {
-#ifdef DEBUG
-    if (_instance != NULL) {
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "ActionHandler(" << this << ")::ActionHandler(" << wm << ")"
-             << endl << " *** _instance allready set: " << _instance << endl;
-    }
-#endif // DEBUG
     _instance = this;
+
+    // Initialize state_to_keycode map
+    for (uint i = 0; i < PScreen::MODIFIER_TO_MASK_NUM; ++i) {
+        uint modifier = PScreen::MODIFIER_TO_MASK[i];
+        _state_to_keycode[modifier] = PScreen::instance()->getKeycodeFromMask(modifier);
+    }
 }
 
 //! @brief ActionHandler destructor
@@ -92,7 +92,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
     bool matched = false;
 
     // determine what type if any of the window object that is focused
-    if (ap.wo != NULL) {
+    if (ap.wo) {
         if (ap.wo->getType() == PWinObj::WO_CLIENT) {
             client = static_cast<Client*>(ap.wo);
             frame = static_cast<Frame*>(client->getParent());
@@ -115,7 +115,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
     list<Action>::const_iterator it = ap.ae.action_list.begin();
     for (; it != ap.ae.action_list.end(); ++it, matched = false) {
         // actions valid for all PWinObjs
-        if ((matched == false) && (ap.wo != NULL)) {
+        if (! matched && ap.wo) {
             matched = true;
             switch (it->getAction()) {
             case ACTION_FOCUS:
@@ -127,6 +127,9 @@ ActionHandler::handleAction(const ActionPerformed &ap)
             case ACTION_FOCUS_DIRECTIONAL:
                 actionFocusDirectional(ap.wo, DirectionType(it->getParamI(0)), it->getParamI(1));
                 break;
+            case ACTION_SEND_KEY:
+                actionSendKey(ap.wo, it->getParamS());
+                break;
             default:
                 matched = false;
                 break;
@@ -134,7 +137,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
         }
 
         // actions valid for Clients and Frames
-        if ((matched == false) && (frame != NULL)) {
+        if (! matched && frame) {
             matched = true;
             switch (it->getAction()) {
             case ACTION_GROUPING_DRAG:
@@ -171,15 +174,15 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 client->kill();
                 break;
             case ACTION_RAISE:
-                if (it->getParamI(0) == true) {
-                    _wm->familyRaiseLower(client, true);
+                if (it->getParamI(0)) {
+                    WindowManager::inst()->familyRaiseLower(client, true);
                 } else {
                     frame->raise();
                 }
                 break;
             case ACTION_LOWER:
-                if (it->getParamI(0) == true) {
-                    _wm->familyRaiseLower(client, false);
+                if (it->getParamI(0)) {
+                    WindowManager::inst()->familyRaiseLower(client, false);
                 } else {
                     frame->lower();
                 }
@@ -216,19 +219,19 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 frame->detachClient(client);
                 break;
             case ACTION_ATTACH_MARKED:
-                _wm->attachMarked(frame);
+                WindowManager::inst()->attachMarked(frame);
                 break;
             case ACTION_ATTACH_CLIENT_IN_NEXT_FRAME:
-                _wm->attachInNextPrevFrame(client, false, true);
+                WindowManager::inst()->attachInNextPrevFrame(client, false, true);
                 break;
             case ACTION_ATTACH_CLIENT_IN_PREV_FRAME:
-                _wm->attachInNextPrevFrame(client, false, false);
+                WindowManager::inst()->attachInNextPrevFrame(client, false, false);
                 break;
             case ACTION_ATTACH_FRAME_IN_NEXT_FRAME:
-                _wm->attachInNextPrevFrame(client, true, true);
+                WindowManager::inst()->attachInNextPrevFrame(client, true, true);
                 break;
             case ACTION_ATTACH_FRAME_IN_PREV_FRAME:
-                _wm->attachInNextPrevFrame(client, true, false);
+                WindowManager::inst()->attachInNextPrevFrame(client, true, false);
                 break;
             default:
                 matched = false;
@@ -238,7 +241,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
 
         // Actions valid for Menus
 #ifdef MENUS
-        if ((matched == false) && (menu != NULL)) {
+        if (! matched && menu) {
             matched = true;
             switch (it->getAction()) {
                 // menu navigation
@@ -256,8 +259,8 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 // it can't be used anymore
                 return;
             case ACTION_MENU_ENTER_SUBMENU:
-                if ((menu->getItemCurr() != NULL) &&
-                        (menu->getItemCurr()->getWORef() != NULL) &&
+                if (menu->getItemCurr() &&
+                        menu->getItemCurr()->getWORef() &&
                         (menu->getItemCurr()->getWORef()->getType() == PWinObj::WO_MENU)) {
                     menu->mapSubmenu(static_cast<PMenu*>(menu->getItemCurr()->getWORef()), true);
                 }
@@ -267,7 +270,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 break;
             case ACTION_CLOSE:
                 menu->unmapAll();
-                _wm->findWOAndFocus(NULL);
+                WindowManager::inst()->findWOAndFocus(NULL);
                 break;
             default:
                 matched = false;
@@ -276,38 +279,28 @@ ActionHandler::handleAction(const ActionPerformed &ap)
         }
 #endif // MENUS
         // actions valid for pdecor
-        if ((matched == false) && (decor != NULL)) {
+        if (! matched && decor) {
             int x_root, y_root;
 
             matched = true;
             switch (it->getAction()) {
             case ACTION_MOVE:
-                // Get root position, first try event then query the pointer
-                if (ap.type == MotionNotify) {
-                    x_root = ap.event.motion->x_root;
-                    y_root = ap.event.motion->y_root;
-                } else if ((ap.type == ButtonPress)
-                           || (ap.type == ButtonRelease)) {
-                    x_root = ap.event.button->x_root;
-                    y_root = ap.event.button->y_root;
-                } else {
-                    PScreen::instance()->getMousePosition(x_root, y_root);
-                }
+                // Get root position, previously event positions was
+                // used however this seems to be error prone on
+                // Xinerama setups
+                PScreen::instance()->getMousePosition(x_root, y_root);
 
                 decor->doMove(x_root, y_root);
                 break;
             case ACTION_CLOSE:
                 decor->unmapWindow();
                 if (decor->getType() == PWinObj::WO_CMD_DIALOG) {
-                    _wm->findWOAndFocus(NULL);
+                    WindowManager::inst()->findWOAndFocus(NULL);
                 }
                 break;
             case ACTION_WARP_TO_WORKSPACE:
                 actionWarpToWorkspace(decor, it->getParamI(0));
                 break;
-            case ACTION_WARP_TO_VIEWPORT:
-                actionWarpToViewport(decor, it->getParamI(0));
-                break;
             default:
                 matched = false;
                 break;
@@ -315,7 +308,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
         }
 
         // Actions valid from everywhere
-        if (matched == false) {
+        if (! matched) {
             matched = true;
             switch (it->getAction()) {
             case ACTION_SET:
@@ -340,41 +333,13 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                                   it->getParamI(1), true);
                 break;
             case ACTION_GOTO_WORKSPACE:
-                // if the event was caused by a motion event ( dragging frame to the
-                // edge ) or enter event ( moving the pointer to the edge ) we'll want
-                // to warp the pointer
-                Workspaces::instance()->gotoWorkspace(it->getParamI(0),
-                                                      ((ap.type == MotionNotify) ||
-                                                       (ap.type == EnterNotify)));
+                // Events caused by a motion event ( dragging frame to
+                // the edge ) or enter event ( moving the pointer to
+                // the edge ) should warp the pointer.
+                actionGotoWorkspace(it->getParamI(0), (ap.type == MotionNotify) || (ap.type == EnterNotify));
                 break;
-            case ACTION_VIEWPORT_MOVE_XY:
-                Workspaces::instance()->getActiveViewport()->move(it->getParamI(0), it->getParamI(1));
-                break;
-            case ACTION_VIEWPORT_MOVE_DRAG:
-                if (ap.type == MotionNotify) {
-                    Workspaces::instance()->getActiveViewport()->moveDrag(ap.event.motion->x_root, ap.event.motion->y_root);
-                }
-                break;
-            case ACTION_VIEWPORT_MOVE_DIRECTION:
-                if (ap.type == KeyPress) {
-                    Workspaces::instance()->getActiveViewport()->moveDirection(DirectionType(it->getParamI(0)), false);
-                } else {
-                    Workspaces::instance()->getActiveViewport()->moveDirection(DirectionType(it->getParamI(0)), true);
-                }
-                break;
-            case ACTION_VIEWPORT_SCROLL:
-                Workspaces::instance()->getActiveViewport()->scroll(it->getParamI(0), it->getParamI(1));
-                break;
-            case ACTION_VIEWPORT_GOTO:
-                Workspaces::instance()->getActiveViewport()->gotoColRow((unsigned) it->getParamI(0), (unsigned) it->getParamI(1));
-                break;
-            case ACTION_SEND_TO_VIEWPORT:
-                actionSendToViewport(decor ? decor : ap.wo,
-                                     it->getParamI(0), it->getParamI(1));
-                break;
-
             case ACTION_FIND_CLIENT:
-                actionFindClient(it->getParamS());
+                actionFindClient(Util::to_wide_str(it->getParamS()));
                 break;
             case ACTION_GOTO_CLIENT_ID:
                 actionGotoClientID(it->getParamI(0));
@@ -389,34 +354,34 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                                ap.type, client ? client : ap.wo);
                 break;
             case ACTION_HIDE_ALL_MENUS:
-                _wm->hideAllMenus();
-                _wm->findWOAndFocus(NULL);
+                WindowManager::inst()->hideAllMenus();
+                WindowManager::inst()->findWOAndFocus(NULL);
                 break;
 #endif // MENUS
             case ACTION_RELOAD:
-                _wm->reload();
-
-                // special case: reload causes the list (ae) to change and therefore
-                // it can't be used anymore
-                return;
+                WindowManager::inst()->reload();
+                break;
             case ACTION_RESTART:
-                _wm->restart();
+                WindowManager::inst()->restart();
                 break;
             case ACTION_RESTART_OTHER:
                 if (it->getParamS().size())
-                    _wm->restart(it->getParamS());
+                    WindowManager::inst()->restart(it->getParamS());
                 break;
             case ACTION_EXIT:
-                _wm->shutdown();
+                WindowManager::inst()->shutdown();
                 break;
             case ACTION_SHOW_CMD_DIALOG:
-                if (_wm->getCmdDialog()->isMapped()) {
-                    _wm->getCmdDialog()->unmapWindow();
+                if (WindowManager::inst()->getCmdDialog()->isMapped()) {
+                    WindowManager::inst()->getCmdDialog()->unmapWindow();
                 } else {
-                    _wm->getCmdDialog()->mapCentered(it->getParamS(), true,
+                    WindowManager::inst()->getCmdDialog()->mapCentered(it->getParamS(), true,
                                                      frame ? frame : ap.wo);
                 }
                 break;
+            case ACTION_HIDE_WORKSPACE_INDICATOR:
+                WindowManager::inst()->getWorkspaceIndicator()->unmapWindow();
+                break;
             default:
                 matched = false;
                 break;
@@ -435,7 +400,7 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
     bool matched = false;
 
     // check for frame actions
-    if ((matched == false) && (frame != NULL)) {
+    if (! matched && frame) {
         matched = true;
         switch (action.getParamI(0)) {
         case ACTION_STATE_MAXIMIZED:
@@ -482,7 +447,7 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
             frame->setDecorOverride(sa, action.getParamS());
             break;
         case ACTION_STATE_TITLE:
-            frame->setStateTitle(sa, client, action.getParamS());
+            frame->setStateTitle(sa, client, Util::to_wide_str(action.getParamS()));
             break;
         default:
             matched = false;
@@ -491,7 +456,7 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
     }
 
     // check for menu actions
-    if ((matched == false) && (wo != NULL) &&
+    if (! matched && wo &&
             (wo->getType() == PWinObj::WO_MENU)) {
         matched = true;
         switch (action.getParamI(0)) {
@@ -504,16 +469,16 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
         }
     }
 
-    if (matched == false) {
+    if (! matched) {
         matched = true;
         switch (action.getParamI(0)) {
 #ifdef HARBOUR
         case ACTION_STATE_HARBOUR_HIDDEN:
-            _wm->getHarbour()->setStateHidden(sa);
+            WindowManager::inst()->getHarbour()->setStateHidden(sa);
             break;
 #endif // HARBOUR
         case ACTION_STATE_GLOBAL_GROUPING:
-            _wm->setStateGlobalGrouping(sa);
+            WindowManager::inst()->setStateGlobalGrouping(sa);
             break;
         default:
             matched = false;
@@ -538,7 +503,7 @@ ActionEvent*
 ActionHandler::findMouseAction(uint button, uint state, MouseEventType type,
                                std::list<ActionEvent> *actions)
 {
-    if (actions == NULL)
+    if (! actions)
         return NULL;
 
     state &= ~PScreen::instance()->getNumLock() &
@@ -560,7 +525,7 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type,
 
 //! @brief Searches for a client matching titles and makes it visible
 void
-ActionHandler::actionFindClient(const std::string &title)
+ActionHandler::actionFindClient(const std::wstring &title)
 {
     Client *client = findClientFromTitle(title);
     if (client) {
@@ -568,6 +533,22 @@ ActionHandler::actionFindClient(const std::string &title)
     }
 }
 
+//! @brief Goto workspace
+//! @param workspace Workspace to got to
+//! @param warp If true, warp pointer as well
+void
+ActionHandler::actionGotoWorkspace(uint workspace, bool warp)
+{
+    Workspaces::instance()->gotoWorkspace(workspace, warp);
+
+    if (Config::instance()->getShowWorkspaceIndicator() > 0) {
+        // Show workspace indicator if requested
+        WindowManager::inst()->getWorkspaceIndicator()->render();
+        WindowManager::inst()->getWorkspaceIndicator()->mapWindowRaised();
+        WindowManager::inst()->getWorkspaceIndicator()->updateHideTimer(Config::instance()->getShowWorkspaceIndicator());
+    }
+}
+
 //! @brief Focus client with id.
 //! @param id Client id.
 void
@@ -616,7 +597,7 @@ void
 ActionHandler::actionWarpToWorkspace(PDecor *decor, uint direction)
 {
     // actually did move
-    if (Workspaces::instance()->gotoWorkspace(DirectionType(direction), true) == true) {
+    if (Workspaces::instance()->gotoWorkspace(DirectionType(direction), true)) {
         int x, y;
         PScreen::instance()->getMousePosition(x, y);
 
@@ -626,60 +607,6 @@ ActionHandler::actionWarpToWorkspace(PDecor *decor, uint direction)
     }
 }
 
-//! @brief
-void
-ActionHandler::actionWarpToViewport(PDecor *decor, uint direction)
-{
-    Viewport *vp = Workspaces::instance()->getActiveViewport();
-
-    // actually did move
-    if (vp->moveDirection(DirectionType(direction)) == true) {
-        int x, y;
-        PScreen::instance()->getMousePosition(x, y);
-
-        decor->move(decor->getClickX() + x - decor->getPointerX(),
-                    decor->getClickY() + y - decor->getPointerY());
-    }
-}
-
-//! @brief Sends PWinObj to viewport col/row
-//! @param wo Pointer to PWinObj to move.
-//! @param col_off Column to send PWinObj to.
-//! @param row_off Row to send PWinObj to.
-void
-ActionHandler::actionSendToViewport(PWinObj *wo, int col_off, int row_off)
-{
-    Viewport *vp = Workspaces::instance()->getActiveViewport();
-
-    // Get current position
-    int col = vp->getCol(wo);
-    int row = vp->getRow(wo);
-
-    // Get new column
-    int col_new = col_off;
-    int row_new = row_off;
-
-    // Validate position
-    if ((col_new < 0) || (col_new >= static_cast<int>(vp->getCols()))) {
-        col_new = col;
-    }
-    if ((row_new < 0) || (row_new >= static_cast<int>(vp->getRows()))) {
-        row_new = row;
-    }
-
-    // Get offset from current viewport to position.
-    int x = wo->getX() + vp->getX() - (col * PScreen::instance()->getWidth());
-    int y = wo->getY() + vp->getY() - (row * PScreen::instance()->getHeight());
-
-    // Add offset to new column and row
-    x += (col_new * PScreen::instance()->getWidth()) - vp->getX();
-    y += (row_new * PScreen::instance()->getHeight()) - vp->getY();
-
-    // Move the window and make sure it is visible.
-    wo->move(x, y);
-    vp->makeWOInsideVirtual(wo);
-}
-
 //! @brief Tries to find the next/prev frame relative to the focused client
 void
 ActionHandler::actionFocusToggle(uint button, uint raise, int off,
@@ -701,13 +628,13 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
     }
 
     // unable to grab keyboard
-    if (PScreen::instance()->grabKeyboard(PScreen::instance()->getRoot()) == false) {
+    if (! PScreen::instance()->grabKeyboard(PScreen::instance()->getRoot())) {
         return;
     }
 
     // find the focused window object
     PWinObj *fo_wo = NULL;
-    if (PWinObj::getFocusedPWinObj() != NULL) {
+    if (PWinObj::getFocusedPWinObj()) {
         if (PWinObj::getFocusedPWinObj()->getType() == PWinObj::WO_CLIENT) {
             fo_wo = PWinObj::getFocusedPWinObj()->getParent();
 
@@ -749,9 +676,9 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
     bool was_iconified = false;
 
     while (cycling) {
-        if (fo_wo != NULL) {
+        if (fo_wo) {
             fo_wo->setFocused(true);
-            if (Raise(raise) == ALWAYS_RAISE)	{
+            if (Raise(raise) == ALWAYS_RAISE) {
                 // Make sure it's not iconified if raise is on.
                 if (fo_wo->isIconified()) {
                     was_iconified = true;
@@ -764,7 +691,7 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
         XMaskEvent(PScreen::instance()->getDpy(), KeyPressMask|KeyReleaseMask, &ev);
         if (ev.type == KeyPress) {
             if (ev.xkey.keycode == button) {
-                if (fo_wo != NULL) {
+                if (fo_wo) {
                     // Restore iconified state
                     if (was_iconified) {
                         was_iconified = false;
@@ -816,14 +743,63 @@ ActionHandler::actionFocusDirectional(PWinObj *wo, DirectionType dir, bool raise
     PWinObj *wo_focus =
         Workspaces::instance()->findDirectional(wo, dir, SKIP_FOCUS_TOGGLE);
 
-    if (wo_focus != NULL) {
-        if (raise == true) {
+    if (wo_focus) {
+        if (raise) {
             wo_focus->raise();
         }
         wo_focus->giveInputFocus();
     }
 }
 
+/**
+ * Parse key information and send to wo.
+ *
+ * To handle this smoothly first all state modifiers are pressed one
+ * by one adding to the state, then the real key is pressed and
+ * released, then the state modifiers are released.
+ *
+ * @param win Window to send key to.
+ * @param key_str String definition of key to send.
+ * @return true if key was parsed ok, no validation of sending is done.
+ */
+bool
+ActionHandler::actionSendKey(PWinObj *wo, const std::string &key_str)
+{
+    uint mod, key;
+    if (! Config::instance()->parseKey(key_str, mod, key)) {
+        return false;
+    }
+
+    XEvent ev;
+    initSendKeyEvent(ev, wo);
+
+    // Press state modifiers
+    map<uint, uint>::iterator it;
+    for (it = _state_to_keycode.begin(); it != _state_to_keycode.end(); ++it) {
+        if (mod & it->first) {
+            ev.xkey.keycode = it->second;
+            XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+            ev.xkey.state |= it->first;
+        }
+    }
+
+    // Send press and release of main key
+    ev.xkey.keycode = key;
+    XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+    ev.type = KeyRelease;
+    XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+
+    // Release state modifiers
+    for (it = _state_to_keycode.begin(); it != _state_to_keycode.end(); ++it) {
+        if (mod & it->first) {
+            ev.xkey.keycode = it->second;
+            XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+            ev.xkey.state &= ~it->first;
+        }
+    }
+
+    return true;
+}
 
 #ifdef MENUS
 //! @brief Toggles visibility of menu.
@@ -835,8 +811,8 @@ void
 ActionHandler::actionShowMenu(const std::string &name, bool stick,
                               uint e_type, PWinObj *wo_ref)
 {
-    PMenu *menu = _wm->getMenu(name);
-    if (menu == NULL) {
+    PMenu *menu = WindowManager::inst()->getMenu(name);
+    if (! menu) {
         return;
     }
 
@@ -877,7 +853,7 @@ ActionHandler::createNextPrevMenu(bool show_iconified)
 {
     ActionEvent ae; // empty ae, used when inserting
     PMenu *menu =
-        new PMenu(PScreen::instance()->getDpy(), _wm->getTheme(), "Windows",
+        new PMenu(PScreen::instance()->getDpy(), WindowManager::inst()->getTheme(), L"Windows",
                   "" /* Empty name*/);
 
     Frame *fr;
@@ -886,7 +862,7 @@ ActionHandler::createNextPrevMenu(bool show_iconified)
         fr = static_cast<Frame*>(*f_it);
         if (createMenuInclude(fr, show_iconified)) {
             menu->insert(static_cast<Client*>(fr->getActiveChild())->getTitle()->getVisible(),
-                         ae, fr);
+                         ae, fr, static_cast<Client*>(fr->getActiveChild())->getIcon());
         }
     }
 
@@ -899,17 +875,16 @@ PMenu*
 ActionHandler::createMRUMenu(bool show_iconified)
 {
     ActionEvent ae; // empty ae, used when inserting
-    PMenu *menu =
-        new PMenu(PScreen::instance()->getDpy(), _wm->getTheme(), "MRU Windows",
-                  "" /* Empty name */);
+    PMenu *menu = new PMenu(PScreen::instance()->getDpy(),
+            WindowManager::inst()->getTheme(), L"MRU Windows", "" /* Empty name */);
 
     Frame *fr;
-    list<PWinObj*>::reverse_iterator f_it = _wm->mru_rbegin();
-    for (; f_it != _wm->mru_rend(); ++f_it) {
+    list<PWinObj*>::reverse_iterator f_it = WindowManager::inst()->mru_rbegin();
+    for (; f_it != WindowManager::inst()->mru_rend(); ++f_it) {
         fr = static_cast<Frame*>(*f_it);
         if (createMenuInclude(fr, show_iconified)) {
             menu->insert(static_cast<Client*>(fr->getActiveChild())->getTitle()->getVisible(),
-                         ae, fr);
+                         ae, fr, static_cast<Client*>(fr->getActiveChild())->getIcon());
         }
     }
 
@@ -925,23 +900,24 @@ ActionHandler::createMenuInclude(Frame *frame, bool show_iconified)
 {
     // Make sure the frame is mapped, or on the correct workspace if
     // it's iconified. Also make sure it's possible to give it focus
-    // and should not be skipped
-    if ((frame->isMapped()
-            || (show_iconified && frame->isIconified()
-                && (frame->isSticky()
-                    || frame->getWorkspace() == Workspaces::instance()->getActive())))
-            && frame->isFocusable()
-            && Workspaces::instance()->getActiveViewport()->isInside(frame)
-            && !frame->isSkip(SKIP_FOCUS_TOGGLE)) {
-        return true;
-    }
-
-    return false;
+    // and should not be skipped. The condition is rather complex, so
+    // we split it up for readability.
+    
+    // focw == frame on current workspace
+    bool focw = frame->isSticky() || frame->getWorkspace() == Workspaces::instance()->getActive();
+    
+    // ibs == iconified but should be shown
+    bool ibs = show_iconified && frame->isIconified() && focw;
+    
+    // mos == mapped or shown nonetheless 
+    bool mos = frame->isMapped() || ibs;
+    
+    return ! frame->isSkip(SKIP_FOCUS_TOGGLE) && frame->isFocusable() && mos;
 }
 
 //! @brief Searches the client list for a client with a title matching title
 Client*
-ActionHandler::findClientFromTitle(const std::string &or_title)
+ActionHandler::findClientFromTitle(const std::wstring &or_title)
 {
     RegexString o_rs;
 
@@ -973,7 +949,7 @@ ActionHandler::gotoClient(Client *client)
     }
 
     // Make sure it's visible
-    if (!frame->isSticky()
+    if (! frame->isSticky()
         && (frame->getWorkspace() != Workspaces::instance()->getActive())) {
         Workspaces::instance()->setWorkspace(frame->getWorkspace(), false);
     }
@@ -982,13 +958,30 @@ ActionHandler::gotoClient(Client *client)
         frame->mapWindow();
     }
 
-    // Make sure it inside Viewport
-    if (! Workspaces::instance()->getActiveViewport()->isInside(frame)) {
-        Workspaces::instance()->getActiveViewport()->moveToWO(frame);
-    }
-
     // Activate it within the Frame.
     frame->activateChild(client);
     frame->raise();
     frame->giveInputFocus();
 }
+
+/**
+ * Initilalize XEvent for sending KeyPress/KeyRelease events.
+ *
+ * @param ev Reference to event.
+ * @param wo PWinObj key event is aimed for.
+ */
+void
+ActionHandler::initSendKeyEvent(XEvent &ev, PWinObj *wo)
+{
+    ev.type = KeyPress;
+    ev.xkey.display = PScreen::instance()->getDpy();
+    ev.xkey.root = PScreen::instance()->getRoot();
+    ev.xkey.window = wo->getWindow();
+    ev.xkey.time = CurrentTime;
+    ev.xkey.x = 1;
+    ev.xkey.y = 1;
+    PScreen::instance()->getMousePosition(ev.xkey.x_root, ev.xkey.y_root);
+    ev.xkey.same_screen = True;
+    ev.xkey.type = KeyPress;
+    ev.xkey.state = 0;
+}
