@@ -1,6 +1,6 @@
 //
 // Client.cc for pekwm
-// Copyright (C) 2002-2006 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2008 Claes Nästén <me{@}pekdon{.}net>
 //
 // client.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -9,14 +9,26 @@
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id: Client.cc 143 2007-05-28 13:36:34Z me@pekdon.net $
-//
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
+#include <cstdio>
+#include <iostream>
+
+extern "C" {
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <X11/Xutil.h>
+#ifdef HAVE_SHAPE
+#include <X11/extensions/shape.h>
+#endif // HAVE_SHAPE
+}
+
 #include "PWinObj.hh"
 #include "PDecor.hh" // PDecor::TitleItem
 #include "Client.hh"
-
 #include "PScreen.hh"
 #include "Config.hh"
 #include "KeyGrabber.hh"
@@ -24,41 +36,31 @@
 #include "AutoProperties.hh"
 #include "Frame.hh"
 #include "Workspaces.hh"
-#include "Viewport.hh"
 #include "WindowManager.hh"
+#include "PTexturePlain.hh"
+#include "PImageNativeIcon.hh"
+#include "TextureHandler.hh"
 
 #ifdef MENUS
 #include "PMenu.hh"
 #include "WORefMenu.hh"
 #endif // MENUS
 
-#include <cstdio>
-
-extern "C" {
-#include <X11/Xatom.h>
-#ifdef HAVE_SHAPE
-#include <X11/extensions/shape.h>
-#endif // HAVE_SHAPE
-}
-
-#ifdef DEBUG
-#include <iostream>
 using std::cerr;
 using std::endl;
-#endif // DEBUG
-
-using std::vector;
-using std::list;
 using std::find;
+using std::list;
 using std::string;
+using std::vector;
+using std::wstring;
 
 list<Client*> Client::_client_list = list<Client*>();
 vector<uint> Client::_clientid_list = vector<uint>();
 
-Client::Client(WindowManager *w, Window new_client, bool is_new)
-    : PWinObj(w->getScreen()->getDpy()),
-      _wm(w), _id(0), _size(NULL),
-      _transient(None), _strut(NULL),
+Client::Client(Window new_client, bool is_new)
+    : PWinObj(WindowManager::inst()->getScreen()->getDpy()),
+      _id(0), _size(NULL),
+      _transient(None), _strut(NULL), _icon(NULL),
       _class_hint(NULL),
       _alive(false), _marked(false),
       _send_focus_message(false), _send_close_message(false),
@@ -80,7 +82,7 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     _title.setId(_id);
     _title.infoAdd(PDecor::TitleItem::INFO_ID);
 
-    if (!validate()) {
+    if (! validate()) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Couldn't validate client." << endl;
@@ -90,7 +92,7 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     }
 
     XWindowAttributes attr;
-    if (!XGetWindowAttributes(_dpy, _window, &attr)) {
+    if (! XGetWindowAttributes(_dpy, _window, &attr)) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Client died." << endl;
@@ -140,20 +142,21 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     _class_hint->title = _title.getReal();
 
     // Get Autoproperties before EWMH as we need the cfg_deny property.
-    AutoProperty *ap = readAutoprops(_wm->isStartup()
+    AutoProperty *ap = readAutoprops(WindowManager::inst()->isStartup()
                                      ? APPLY_ON_NEW : APPLY_ON_START);
 
     readMwmHints(); // read atoms
     readEwmhHints();
     readPekwmHints();
+    readIcon();
     getWMProtocols();
 
     ulong initial_state = NormalState;
     XWMHints* hints = XGetWMHints(_dpy, _window);
-    if (hints != NULL) {
+    if (hints) {
         // get the input focus mode
         if (hints->flags&InputHint) { // FIXME: More logic needed
-          _wm_hints_input = hints->input;
+            _wm_hints_input = hints->input;
         }
 
         // Get initial state of the window
@@ -165,13 +168,18 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     }
 
     // Set state either specified in hint
-    setWmState(initial_state);
-
-    // are we iconified
     if (getWmState() == IconicState) {
         _iconified = true;
     }
 
+    if (_iconified || initial_state == IconicState) {
+        _iconified = true;
+        _mapped = true;
+        unmapWindow();
+    } else {
+        setWmState(initial_state);
+    }
+
     // grab keybindings and mousebutton actions
     KeyGrabber::instance()->grabKeys(_window);
     grabButtons();
@@ -183,7 +191,7 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     // If pekwm allready is running, check against autoproperty then
     // tagged frame. If starting up, check if it has a FRAME_ID and if not
     // use autoproperties.
-    if (_wm->isStartup()) {
+    if (WindowManager::inst()->isStartup()) {
         // Check for tagged frame
         Frame *frame = Frame::getTagFrame();
         if (frame && frame->isMapped()) {
@@ -219,12 +227,12 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     if (ap) {
         applyAutoprops(ap);
 
-        if (do_autogroup && !_parent && (ap->group_size >= 0)) {
-            Frame* frame = _wm->findGroup(ap);
+        if (do_autogroup && ! _parent && (ap->group_size >= 0)) {
+            Frame* frame = WindowManager::inst()->findGroup(ap);
             if (frame) {
                 frame->addChild(this);
 
-                if (ap->group_behind == false) {
+                if (! ap->group_behind) {
                     frame->activateChild(this);
                     do_focus = frame->isFocused();
                 }
@@ -241,17 +249,19 @@ Client::Client(WindowManager *w, Window new_client, bool is_new)
     PScreen::instance()->ungrabServer(true);
 
     // if we don't have a frame allready, create a new one
-    if (_parent == NULL) {
-        _parent = new Frame(_wm, this, ap);
+    if (! _parent) {
+        _parent = new Frame(this, ap);
     }
 
-    // make sure the window is mapped, this is done after it has been
+    // Make sure the window is mapped, this is done after it has been
     // added to the decor/frame as otherwise IsViewable state won't
     // be correct and we don't know wheter or not to place the window
-    PWinObj::mapWindow();
+    if (! _iconified) {
+        PWinObj::mapWindow();
+    }
 
     // Let us hear what autoproperties has to say about focusing
-    if (is_new && (ap != NULL) && ap->isMask(AP_FOCUS_NEW)) {
+    if (is_new && ap && ap->isMask(AP_FOCUS_NEW)) {
         do_focus = ap->focus_new;
     }
 
@@ -294,13 +304,13 @@ Client::~Client(void)
 #ifdef MENUS
     WORefMenu *wo_ref_menu;
 
-    wo_ref_menu = static_cast<WORefMenu*>(_wm->getMenu("WINDOW"));
+    wo_ref_menu = static_cast<WORefMenu*>(WindowManager::inst()->getMenu("WINDOW"));
     if (this == wo_ref_menu->getWORef()) {
         wo_ref_menu->setWORef(NULL);
         wo_ref_menu->unmapAll();
     }
 
-    wo_ref_menu = static_cast<WORefMenu*>(_wm->getMenu("DECORMENU"));
+    wo_ref_menu = static_cast<WORefMenu*>(WindowManager::inst()->getMenu("DECORMENU"));
     if (this == wo_ref_menu->getWORef()) {
         wo_ref_menu->setWORef(NULL);
         wo_ref_menu->unmapAll();
@@ -317,7 +327,7 @@ Client::~Client(void)
     // Focus the parent if we had focus before
     if (focus && (_transient != None)) {
         Client *trans_cli = findClientFromWindow(_transient);
-        if ((trans_cli != NULL) &&
+        if (trans_cli &&
                 (((Frame*) trans_cli->getParent())->getActiveChild() == trans_cli)) {
             trans_cli->getParent()->giveInputFocus();
         }
@@ -338,8 +348,14 @@ Client::~Client(void)
 
     removeStrutHint();
 
-    if (_class_hint)
+    if (_class_hint) {
         delete _class_hint;
+    }
+
+    if (_icon) {
+        TextureHandler::instance()->returnTexture(_icon);
+        _icon = NULL;
+    }
 
     PScreen::instance()->ungrabServer(true);
 }
@@ -350,8 +366,9 @@ Client::~Client(void)
 void
 Client::mapWindow(void)
 {
-    if (_mapped)
+    if (_mapped) {
         return;
+    }
 
     if (_iconified) {
         _iconified = false;
@@ -359,8 +376,10 @@ Client::mapWindow(void)
         updateEwmhStates();
     }
 
-    if(!_transient) // unmap our transient windows if we have any
-        _wm->findTransientsToMapOrUnmap(_window, false);
+    if(! _transient) {
+        // Unmap our transient windows if we have any
+        WindowManager::inst()->findTransientsToMapOrUnmap(_window, false);
+    }
 
     XSelectInput(_dpy, _window, NoEventMask);
     PWinObj::mapWindow();
@@ -373,10 +392,12 @@ Client::mapWindow(void)
 void
 Client::unmapWindow(void)
 {
-    if (!_mapped)
+    if (! _mapped) {
         return;
+    }
 
-    if (_iconified) { // set the state of the window
+    if (_iconified) {
+        // Set the state of the window
         setWmState(IconicState);
         updateEwmhStates();
     }
@@ -391,12 +412,14 @@ Client::unmapWindow(void)
 void
 Client::iconify(void)
 {
-    if (_iconified)
+    if (_iconified) {
         return;
-    _iconified = true;
+    }
 
-    if (!_transient)
-        _wm->findTransientsToMapOrUnmap(_window, true);
+    _iconified = true;
+    if (! _transient) {
+        WindowManager::inst()->findTransientsToMapOrUnmap(_window, true);
+    }
 
     unmapWindow();
 }
@@ -412,7 +435,7 @@ Client::stick(void)
 
 //! @brief Update the position variables.
 void
-Client::move(int x, int y, bool do_virtual)
+Client::move(int x, int y)
 {
     bool request = ((_gm.x != x) || (_gm.y != y));
 
@@ -424,15 +447,6 @@ Client::move(int x, int y, bool do_virtual)
     }
 }
 
-//! @brief
-void
-Client::moveVirtual(int x, int y)
-{
-    AtomUtil::setPosition(_window,
-                          PekwmAtoms::instance()->getAtom(PEKWM_FRAME_VPOS),
-                          x, y);
-}
-
 //! @brief Resizes the client window to specified size.
 void
 Client::resize(uint width, uint height)
@@ -446,6 +460,20 @@ Client::resize(uint width, uint height)
     }
 }
 
+//! @brief Move and resizes the client window to specified size.
+void
+Client::moveResize(int x, int y, uint width, uint height)
+{
+    bool request = ((_gm.x != x) || (_gm.y != y)
+                  || (_gm.width != width) || (_gm.height != height));
+
+    PWinObj::resize(width, height);
+
+    if (request) {
+        configureRequestSend();
+    }
+}
+
 //! @brief Sets the workspce and updates the _NET_WM_DESKTOP hint.
 void
 Client::setWorkspace(uint workspace)
@@ -470,18 +498,17 @@ Client::setWorkspace(uint workspace)
 bool
 Client::giveInputFocus(void)
 {
-  if (_wm_hints_input) {
-    return PWinObj::giveInputFocus();
-
-  } else if (_send_focus_message) {
-    sendXMessage(_window,
-                 IcccmAtoms::instance()->getAtom(WM_PROTOCOLS), NoEventMask,
-                 IcccmAtoms::instance()->getAtom(WM_TAKE_FOCUS),
-                 PScreen::instance()->getLastEventTime());
-    return true;
-  } else {
-    return true;
-  }
+    if (_wm_hints_input) {
+        return PWinObj::giveInputFocus();
+    } else if (_send_focus_message) {
+        sendXMessage(_window,
+                    IcccmAtoms::instance()->getAtom(WM_PROTOCOLS), NoEventMask,
+                    IcccmAtoms::instance()->getAtom(WM_TAKE_FOCUS),
+                    PScreen::instance()->getLastEventTime());
+        return true;
+    } else {
+        return true;
+    }
 }
 
 //! @brief Reparents and sets _parent member, filtering unmap events
@@ -490,6 +517,8 @@ Client::reparent(PWinObj *parent, int x, int y)
 {
     XSelectInput(_dpy, _window, NoEventMask);
     PWinObj::reparent(parent, x, y);
+    _gm.x = parent->getX() + x;
+    _gm.y = parent->getY() + y;
     XSelectInput(_dpy, _window,
                  PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
 }
@@ -501,6 +530,15 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
     if ((ev->window != ev->event) && (ev->send_event != true)) {
         return NULL;
     }
+
+    // ICCCM 4.1.4 advices the window manager to trigger the transition to
+    // Withdrawn state on real and synthetic UnmapNotify events.
+    setWmState(WithdrawnState);
+
+    // Extended Window Manager Hints 1.3 specifies that a window manager
+    // should remove the _NET_WM_STATE property when a window is withdrawn.
+    AtomUtil::unsetProperty(_window, EwmhAtoms::instance()->getAtom(STATE));
+
 #ifdef DEBUG
     cerr << __FILE__ << "@" << __LINE__ << ": "
          << "Client(" << this << ")::handleUnmapEvent(" << ev << ")" << endl
@@ -508,7 +546,7 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
 #endif // DEBUG
 
     // FIXME: Listen mask should change as this doesn't work?
-    //_alive = false;
+    _alive = false;
     delete this;
 
     return NULL;
@@ -544,7 +582,7 @@ Client*
 Client::findClientFromWindow(Window win)
 {
     // Validate input window.
-    if ((win == None) || (win == PScreen::instance()->getRoot())) {
+    if (! win || win == PScreen::instance()->getRoot()) {
         return NULL;
     }
 
@@ -649,7 +687,7 @@ Client::grabButtons(void)
 void
 Client::setStateCfgDeny(StateAction sa, uint deny)
 {
-    if (ActionUtil::needToggle(sa, _state.cfg_deny&deny) == false) {
+    if (! ActionUtil::needToggle(sa, _state.cfg_deny&deny)) {
         return;
     }
 
@@ -666,16 +704,18 @@ Client::readClassRoleHints(void)
     // class hint
     XClassHint class_hint;
     if (XGetClassHint(_dpy, _window, &class_hint)) {
-        _class_hint->h_name = class_hint.res_name;
-        _class_hint->h_class = class_hint.res_class;
+        _class_hint->h_name = Util::to_wide_str(class_hint.res_name);
+        _class_hint->h_class = Util::to_wide_str(class_hint.res_class);
         XFree(class_hint.res_name);
         XFree(class_hint.res_class);
     }
 
     // wm window role
-    _class_hint->h_role = "";
-    AtomUtil::getString(_window, IcccmAtoms::instance()->getAtom(WM_WINDOW_ROLE),
-                        _class_hint->h_role);
+    string role;
+    AtomUtil::getString(_window,
+                        IcccmAtoms::instance()->getAtom(WM_WINDOW_ROLE),  role);
+
+    _class_hint->h_role = Util::to_wide_str(role);
 }
 
 //! @brief Loads the Clients state from EWMH atoms.
@@ -698,72 +738,54 @@ Client::readEwmhHints(void)
     int items;
     Atom *atoms = NULL;
 
-    bool found_window_type = false;
-    atoms = (Atom*)
-            AtomUtil::getEwmhPropData(_window, ewmh->getAtom(WINDOW_TYPE),
-                                      XA_ATOM, items);
+    EwmhAtomName window_type = WINDOW_TYPE;
+    atoms = (Atom*) AtomUtil::getEwmhPropData(_window,
+                                              ewmh->getAtom(WINDOW_TYPE),
+                                              XA_ATOM, items);
     if (atoms) {
-        for (int i = 0; i < items; ++i) {
-            found_window_type = true;
-
-            if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DESKTOP)) {
-                // desktop windows, make it the same size as the screen and place
-                // it below all windows withouth any decorations, also make it sticky
-                _gm.x = _gm.y = 0;
-                _gm.width = PScreen::instance()->getWidth();
-                _gm.height = PScreen::instance()->getHeight();
-                setTitlebar(false);
-                setBorder(false);
-                _sticky = true;
-                _state.placed = true;
-                _state.skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE|SKIP_SNAP;
-                _layer = LAYER_DESKTOP;
-                break;
-            } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DOCK)) {
-                setTitlebar(false);
-                setBorder(false);
-                _sticky = true;
-                _state.skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE;
-                _layer = LAYER_DOCK;
-                break;
-            } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_TOOLBAR)) {
-                setTitlebar(false);
-                setBorder(false);
-                _state.skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE;
-                break;
-            } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_MENU)) {
-                _layer = LAYER_MENU;
-                _state.skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE|SKIP_SNAP;
-                break;
-// 			} else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_UTILITY)) {
-// 			}
-// 			} else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DIALOG)) {
-// 			}
-            } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_SPLASH)) {
-                setTitlebar(false);
-                setBorder(false);
-            } else {
-                found_window_type = false;
-            }
+      for (int i = 0; window_type == WINDOW_TYPE && i < items; ++i) {
+        if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DESKTOP)) {
+          window_type = WINDOW_TYPE_DESKTOP;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DOCK)) {
+          window_type = WINDOW_TYPE_DOCK;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_TOOLBAR)) {
+          window_type = WINDOW_TYPE_TOOLBAR;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_MENU)) {
+          window_type = WINDOW_TYPE_MENU;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_UTILITY)) {
+          window_type = WINDOW_TYPE_UTILITY;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_DIALOG)) {
+          window_type = WINDOW_TYPE_DIALOG;
+        } else if (atoms[i] == ewmh->getAtom(WINDOW_TYPE_SPLASH)) {
+          window_type = WINDOW_TYPE_SPLASH;
         }
+      }
 
-        XFree(atoms);
+      XFree(atoms);
     }
 
-    if (!found_window_type) {
-        Atom type = ewmh->getAtom(WINDOW_TYPE_NORMAL);
-        AtomUtil::setAtom(_window, ewmh->getAtom(WINDOW_TYPE), type);
+    // Set window type to WINDOW_TYPE_NORMAL if it did not match
+    if (window_type == WINDOW_TYPE) {
+      window_type = WINDOW_TYPE_NORMAL;
+      AtomUtil::setAtom(_window, ewmh->getAtom(WINDOW_TYPE),
+                        ewmh->getAtom(WINDOW_TYPE_NORMAL));
+    }
+     
+    // Apply autoproperties for window type
+    AutoProperty *auto_property = AutoProperties::instance()->findWindowTypeProperty(window_type);
+    if (auto_property) {
+      applyAutoprops(auto_property);
     }
 
     // The _NET_WM_STATE overides the _NET_WM_TYPE
     NetWMStates win_states;
-    if (getEwmhStates(win_states) == true) {
+    if (getEwmhStates(win_states)) {
         if (win_states.hidden) _iconified = true;
         if (win_states.shaded) _state.shaded = true;
         if (win_states.max_vert) _state.maximized_vert = true;
         if (win_states.max_horz) _state.maximized_horz = true;
-        if (win_states.skip_taskbar) _state.skip_taskbar = true;
-        if (win_states.skip_pager) _state.skip_pager = true;
+        if (win_states.skip_taskbar) _state.skip |= SKIP_TASKBAR;
+        if (win_states.skip_pager) _state.skip |= SKIP_PAGER;
         if (win_states.sticky) _sticky = true;
         if (win_states.above) _layer = LAYER_BELOW;
         if (win_states.below) _layer = LAYER_ABOVE_DOCK;
@@ -780,30 +802,31 @@ Client::readMwmHints(void)
 {
     MwmHints *mwm_hints = getMwmHints(_window);
 
-    if (mwm_hints != NULL) {
-
+    if (mwm_hints) {
         if (mwm_hints->flags&MWM_HINTS_FUNCTIONS) {
-            if (mwm_hints->functions&MWM_FUNC_ALL != 0) {
-                if (mwm_hints->functions&MWM_FUNC_RESIZE == 0)
-                    _actions.resize = false;
-                if (mwm_hints->functions&MWM_FUNC_MOVE == 0)
-                    _actions.move = false;
-                if (mwm_hints->functions&MWM_FUNC_ICONIFY == 0)
-                    _actions.minimize = false;
-                if (mwm_hints->functions&MWM_FUNC_CLOSE == 0)
-                    _actions.close = false;
-                if (mwm_hints->functions&MWM_FUNC_MAXIMIZE == 0) {
-                    _actions.maximize_vert = true;
-                    _actions.maximize_horz = true;
-                }
-            }
+            bool state = ! (mwm_hints->functions&MWM_FUNC_ALL);
+
+            _actions.resize   = (mwm_hints->functions&MWM_FUNC_RESIZE)  ? state : ! state;
+            _actions.move     = (mwm_hints->functions&MWM_FUNC_MOVE)    ? state : ! state;
+            _actions.minimize = (mwm_hints->functions&MWM_FUNC_ICONIFY) ? state : ! state;
+            _actions.close    = (mwm_hints->functions&MWM_FUNC_CLOSE)   ? state : ! state;
+            _actions.maximize_vert = (mwm_hints->functions&MWM_FUNC_MAXIMIZE) ? state : ! state;
+            _actions.maximize_horz = (mwm_hints->functions&MWM_FUNC_MAXIMIZE) ? state : ! state;
         }
 
-        if (mwm_hints->flags&MWM_HINTS_DECORATIONS) {
-            if (!mwm_hints->decorations&MWM_DECOR_ALL) {
-                setTitlebar(mwm_hints->decorations&MWM_DECOR_TITLE);
-                setBorder(mwm_hints->decorations&MWM_DECOR_BORDER);
-            }
+        // Check decoration flags
+        if (mwm_hints->flags & MWM_HINTS_DECORATIONS
+            && ! (mwm_hints->decorations & MWM_DECOR_ALL)) {
+          if (! (mwm_hints->decorations & MWM_DECOR_TITLE)) {
+            setTitlebar(false);
+          }
+          if (! (mwm_hints->decorations & MWM_DECOR_BORDER)) {
+            setBorder(false);
+          }
+
+          // Do not handle HANDLE, MENU, ICONFIY or MAXIMIZE. Maybe
+          // one should set the allowed actions for the client based
+          // on this but that might be annoying so ignoring these.
         }
 
         XFree(mwm_hints);
@@ -819,10 +842,6 @@ Client::readPekwmHints(void)
 
     PekwmAtoms *atoms = PekwmAtoms::instance();
 
-    // Virtual position overides real position
-    AtomUtil::getPosition(_window, atoms->getAtom(PEKWM_FRAME_VPOS),
-                          _gm.x, _gm.y);
-
     // Get decor state
     if (AtomUtil::getLong(_window, atoms->getAtom(PEKWM_FRAME_DECOR),
                           value)) {
@@ -836,27 +855,54 @@ Client::readPekwmHints(void)
 
     // Get custom title
     if (AtomUtil::getString(_window, atoms->getAtom(PEKWM_TITLE), str)) {
-        _title.setUser(str);
+        _title.setUser(Util::to_wide_str(str));
+    }
+}
+
+//! @brief Read _NET_WM_ICON from client window.
+void
+Client::readIcon(void)
+{
+    PImageNativeIcon *image = new PImageNativeIcon(_dpy);
+    if (image->loadFromWindow(_window)) {
+        if (! _icon) {
+            _icon = new PTextureImage(_dpy);
+            TextureHandler::instance()->referenceTexture(_icon);
+        }
+
+        _icon->setImage(image);
+    } else {
+        if (image) {
+            delete image;
+        }
+
+        if (_icon) {
+            TextureHandler::instance()->returnTexture(_icon);
+            _icon = NULL;
+        }
     }
 }
 
 //! @brief Tries to find a AutoProp for the current client.
 //! @param type Defaults to 0
+//! @return AutoProperty if any is found, else NULL.
 AutoProperty*
 Client::readAutoprops(uint type)
 {
     AutoProperty *data =
         AutoProperties::instance()->findAutoProperty(_class_hint, _workspace, type);
 
-    if ((data == NULL) ||
-            (_transient
-             ? !data->isApplyOn(APPLY_ON_TRANSIENT|APPLY_ON_TRANSIENT_ONLY)
-             : data->isApplyOn(APPLY_ON_TRANSIENT_ONLY))) {
-        return NULL;
+    if (data) {
+        // Make sure transient state matches
+        if (_transient
+                ? data->isApplyOn(APPLY_ON_TRANSIENT|APPLY_ON_TRANSIENT_ONLY)
+                : ! data->isApplyOn(APPLY_ON_TRANSIENT_ONLY)) {
+            applyAutoprops(data);
+        } else {
+            data = NULL;
+        }
     }
 
-    applyAutoprops(data);
-
     return data;
 }
 
@@ -933,33 +979,39 @@ Client::returnClientID(uint id)
 void
 Client::getXClientName(void)
 {
-    string title;
-
-    // read X11 property
-    XTextProperty text_property;
-    if ((XGetWMName(_dpy, _window, &text_property) == False) ||
-            (text_property.value == NULL) || (text_property.nitems == 0)){
-        return;
-    }
-
-    if (text_property.encoding == XA_STRING) {
-        title = (const char*) text_property.value;
+    wstring title;
 
+    if (AtomUtil::getUtf8String(_window,
+                              EwmhAtoms::instance()->getAtom(NET_WM_NAME),
+                              title)) {
     } else {
-        char **list;
-        int num;
-
-        XmbTextPropertyToTextList(_dpy, &text_property, &list, &num);
-        if ((num > 0) && (*list != NULL)) {
-            title = *list;
-            XFreeStringList(list);
+        // read X11 property
+        XTextProperty text_property;
+        if ((XGetWMName(_dpy, _window, &text_property) == False) ||
+                ! text_property.value || (text_property.nitems == 0)){
+            return;
         }
-    }
 
-    XFree(text_property.value);
+        if (text_property.encoding == XA_STRING) {
+            string value(reinterpret_cast<const  char*>(text_property.value));
+            title = Util::to_wide_str(value);
+        } else {
+            char **list;
+            int num;
+
+            XmbTextPropertyToTextList(_dpy, &text_property, &list, &num);
+            if (list && (num > 0)) {
+                string value(*list);
+                title = Util::to_wide_str(value);
+                XFreeStringList(list);
+            }
+        }
+
+        XFree(text_property.value);
+    }
 
     // Mirror it on the visible
-    _title.setCustom("");
+    _title.setCustom(L"");
     _title.setCount(titleFindID(title));
     _title.setReal(title);
 
@@ -974,12 +1026,12 @@ Client::getXClientName(void)
 //! @param title Title to apply rule on.
 //! @return true if rule was applied, else false.
 bool
-Client::titleApplyRule(std::string &title)
+Client::titleApplyRule(std::wstring &title)
 {
     _class_hint->title = title;
     TitleProperty *data =
         AutoProperties::instance()->findTitleProperty (_class_hint);
-    _class_hint->title = "";
+    _class_hint->title = L"";
 
     if (data) {
         return data->getTitleRule ().ed_s (title);
@@ -992,10 +1044,10 @@ Client::titleApplyRule(std::string &title)
 //! @param title Title of client to find ID for.
 //! @return Number of clients with that id.
 uint
-Client::titleFindID(std::string &title)
+Client::titleFindID(std::wstring &title)
 {
     // Do not search for unique IDs if it is not enabled.
-    if (Config::instance()->getClientUniqueName() == false) {
+    if (! Config::instance()->getClientUniqueName()) {
         return 0;
     }
 
@@ -1038,8 +1090,8 @@ Client::getXIconName(void)
     _icon_name = "";
 
     XTextProperty text_property;
-    if (!XGetWMIconName(_dpy, _window, &text_property) ||
-            !text_property.value || !text_property.nitems)
+    if (! XGetWMIconName(_dpy, _window, &text_property) ||
+            ! text_property.value || ! text_property.nitems)
         return;
 
     if (text_property.encoding == XA_STRING) {
@@ -1074,7 +1126,7 @@ Client::setWmState(ulong state)
                     32, PropModeReplace, (uchar*) data, 2);
 }
 
-// If we can't find a _wm->wm_state we're going to have to assume
+// If we can't find a wm_state we're going to have to assume
 // Withdrawn. This is not exactly optimal, since we can't really
 // distinguish between the case where no WM has run yet and when the
 // state was explicitly removed (Clients are allowed to either set the
@@ -1107,7 +1159,7 @@ void
 Client::configureRequestSend(void)
 {
     if (_cfg_request_lock) {
-        return;
+      return;
     }
 
     XConfigureEvent e;
@@ -1263,8 +1315,8 @@ Client::getMwmHints(Window win)
     uchar *udata;
 
     int status =
-        XGetWindowProperty(_dpy, win, _wm->getMwmHintsAtom(),
-                           0L, 20L, False, _wm->getMwmHintsAtom(),
+        XGetWindowProperty(_dpy, win, WindowManager::inst()->getMwmHintsAtom(),
+                           0L, 20L, False, WindowManager::inst()->getMwmHintsAtom(),
                            &real_type, &real_format,
                            &items_read, &items_left,
                            &udata);
@@ -1335,7 +1387,7 @@ Client::getEwmhStates(NetWMStates &win_states)
              AtomUtil::getEwmhPropData(_window, EwmhAtoms::instance()->getAtom(STATE),
                                        XA_ATOM, num);
 
-    if (states != NULL) {
+    if (states) {
         EwmhAtoms *ewmh = EwmhAtoms::instance(); // convenience
 
         for (int i = 0; i < num; ++i) {
@@ -1344,10 +1396,10 @@ Client::getEwmhStates(NetWMStates &win_states)
             } else if (states[i] == ewmh->getAtom(STATE_STICKY)) {
                 win_states.sticky = true;
             } else if (states[i] == ewmh->getAtom(STATE_MAXIMIZED_VERT)
-                       && !isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
                 win_states.max_vert = true;
             } else if (states[i] == ewmh->getAtom(STATE_MAXIMIZED_HORZ)
-                       && !isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
                 win_states.max_horz = true;
             } else if (states[i] == ewmh->getAtom(STATE_SHADED)) {
                 win_states.shaded = true;
@@ -1356,16 +1408,16 @@ Client::getEwmhStates(NetWMStates &win_states)
             } else if (states[i] == ewmh->getAtom(STATE_SKIP_PAGER)) {
                 win_states.skip_pager = true;
             } else if (states[i] == ewmh->getAtom(STATE_HIDDEN)
-                       && !isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
                 win_states.hidden = true;
             } else if (states[i] == ewmh->getAtom(STATE_FULLSCREEN)
-                       && !isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
                 win_states.fullscreen = true;
             } else if (states[i] == ewmh->getAtom(STATE_ABOVE)
-                       && !isCfgDeny(CFG_DENY_STATE_ABOVE)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_ABOVE)) {
                 win_states.above = true;
             } else if (states[i] == ewmh->getAtom(STATE_BELOW)
-                       && !isCfgDeny(CFG_DENY_STATE_BELOW)) {
+                       && ! isCfgDeny(CFG_DENY_STATE_BELOW)) {
                 win_states.below = true;
             }
         }
@@ -1396,9 +1448,9 @@ Client::updateEwmhStates(void)
         states.push_back(ewmh->getAtom(STATE_MAXIMIZED_HORZ));
     if (_state.shaded)
         states.push_back(ewmh->getAtom(STATE_SHADED));
-    if (_state.skip_taskbar)
+    if (isSkip(SKIP_TASKBAR))
         states.push_back(ewmh->getAtom(STATE_SKIP_TASKBAR));
-    if (_state.skip_pager)
+    if (isSkip(SKIP_PAGER))
         states.push_back(ewmh->getAtom(STATE_SKIP_PAGER));
     if (_iconified)
         states.push_back(ewmh->getAtom(STATE_HIDDEN));
@@ -1451,7 +1503,7 @@ Client::getWMProtocols(void)
 void
 Client::getTransientForHint(void)
 {
-    if (!_transient)
+    if (! _transient)
         XGetTransientForHint(_dpy, _window, &_transient);
 }
 
@@ -1467,8 +1519,8 @@ Client::getStrutHint(void)
                                       EwmhAtoms::instance()->getAtom(NET_WM_STRUT),
                                       XA_CARDINAL, num);
 
-    if (strut != NULL) {
-        if (_strut != NULL) {
+    if (strut) {
+        if (_strut) {
             PScreen::instance()->removeStrut(_strut);
         } else {
             _strut = new Strut();
@@ -1479,26 +1531,21 @@ Client::getStrutHint(void)
 
         XFree(strut);
 
-    } else if (_strut != NULL) {
+    } else if (_strut) {
         PScreen::instance()->removeStrut(_strut);
         delete _strut;
         _strut = NULL;
     }
-
-    Workspaces::instance()->getActiveViewport()->hintSetWorkarea();
 }
 
 //! @brief
 void
 Client::removeStrutHint(void)
 {
-    if (_strut == NULL)
+    if (! _strut)
         return;
 
     PScreen::instance()->removeStrut(_strut);
     delete _strut;
     _strut = NULL;
-
-    Workspaces::instance()->getActiveViewport()->hintSetWorkarea();
 }
-
