@@ -1,14 +1,24 @@
 //
 // Frame.cc for pekwm
-// Copyright (C) 2002-2006 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2008 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id: Frame.cc 119 2006-09-29 14:06:52Z me@pekdon.net $
-//
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
+#include <algorithm>
+#include <cstdio>
+#include <functional>
+#include <iostream>
+
+extern "C" {
+#include <X11/Xatom.h>
+}
+
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "Frame.hh"
@@ -18,7 +28,6 @@
 #include "ActionHandler.hh"
 #include "AutoProperties.hh"
 #include "Client.hh"
-#include "Viewport.hh"
 #include "ScreenResources.hh"
 #include "StatusWindow.hh"
 #include "Workspaces.hh"
@@ -34,24 +43,14 @@
 
 #include "Theme.hh"
 
-#include <algorithm>
-#include <functional>
-#include <cstdio> // for snprintf
-
-extern "C" {
-#include <X11/Xatom.h>
-}
-
-#ifdef DEBUG
-#include <iostream>
 using std::cerr;
 using std::endl;
-#endif // DEBUG
-using std::string;
-using std::list;
-using std::vector;
-using std::mem_fun;
 using std::find;
+using std::list;
+using std::mem_fun;
+using std::string;
+using std::vector;
+using std::wstring;
 
 list<Frame*> Frame::_frame_list = list<Frame*>();
 vector<uint> Frame::_frameid_list = vector<uint>();
@@ -60,16 +59,13 @@ Frame* Frame::_tag_frame = NULL;
 bool Frame::_tag_behind = false;
 
 //! @brief Frame constructor
-Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
-    : PDecor(wm->getScreen()->getDpy(), wm->getTheme(),
-             (wm->getAutoProperties()->findDecorProperty(client->getClassHint()) == NULL)
-             ? PDecor::DEFAULT_DECOR_NAME
-             : wm->getAutoProperties()->findDecorProperty(client->getClassHint())->getName()),
-      _wm(wm),
+Frame::Frame(Client *client, AutoProperty *ap)
+    : PDecor(WindowManager::inst()->getScreen()->getDpy(), WindowManager::inst()->getTheme(),
+             Frame::getClientDecorName(client)),
       _id(0), _client(NULL), _class_hint(NULL), _old_decor_state(0)
 {
     // setup basic pointers
-    _scr = _wm->getScreen();
+    _scr = WindowManager::inst()->getScreen();
 
     // PWinObj attributes
     _type = WO_FRAME;
@@ -88,7 +84,7 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
     }
 
     // get unique id of the frame, if the client didn't have an id
-    if (_wm->isStartup() == false) {
+    if (! WindowManager::inst()->isStartup()) {
         long id;
 
         if (AtomUtil::getLong(client->getWindow(),
@@ -112,10 +108,12 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
 
     // Before setting position and size up we make sure the decor state
     // of the client match the decore state of the framewidget
-    if (!client->hasBorder())
+    if (! client->hasBorder()) {
         setBorder(STATE_UNSET);
-    if (!client->hasTitlebar())
+    }
+    if (! client->hasTitlebar()) {
         setTitlebar(STATE_UNSET);
+    }
 
     // We first get the size of the window as it will be needed when placing
     // the window with the help of WinGravity.
@@ -134,7 +132,7 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
     }
 
     // override both position and size with autoproperties
-    if (ap != NULL) {
+    if (ap) {
         if (ap->isMask(AP_FRAME_GEOMETRY|AP_CLIENT_GEOMETRY)) {
             setupAPGeometry(client, ap);
 
@@ -144,24 +142,6 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
             }
         }
 
-        if (ap->isMask(AP_VIEWPORT)) {
-            Viewport *vp = Workspaces::instance()->getViewport(client->getWorkspace());
-            if (vp != NULL) {
-                // get position on that viewport then add viewport base position
-                if (ap->viewport_col < vp->getCols()) {
-                    _gm.x -= vp->getCol(this) * PScreen::instance()->getWidth();
-                    _gm.x += ap->viewport_col * PScreen::instance()->getWidth();
-                }
-                if (ap->viewport_row < vp->getRows()) {
-                    _gm.y -= vp->getRow(this) * PScreen::instance()->getHeight();
-                    _gm.y += ap->viewport_row * PScreen::instance()->getHeight();
-                }
-                move(_gm.x, _gm.y);
-
-                place = false;
-            }
-        }
-
         if (ap->isMask(AP_PLACE_NEW)) {
             place = ap->place_new;
         }
@@ -183,7 +163,7 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
     // I add these to the list before I insert the client into the frame to
     // be able to skip an extra updateClientList
     _frame_list.push_back(this);
-    _wm->addToFrameList(this);
+    WindowManager::inst()->addToFrameList(this);
     Workspaces::instance()->insert(this);
 
     // now insert the client in the frame we created, do not give it focus.
@@ -193,19 +173,9 @@ Frame::Frame(WindowManager *wm, Client *client, AutoProperty *ap)
     // set the window states, shaded, maximized...
     getState(client);
 
-    // FIXME: This should be checked even if viewports are enabled.
-    if ((Config::instance()->getViewportCols() < 2) &&
-            (Config::instance()->getViewportRows() < 2)) {
-        if (_client->hasStrut() == false) {
-            if (fixGeometry()) {
-                move(_gm.x, _gm.y);
-                resize(_gm.width, _gm.height);
-            }
-        }
-    } else {
-        Viewport *vp = Workspaces::instance()->getViewport(_client->getWorkspace());
-        if (vp != NULL) {
-            vp->makeWOInsideVirtual(this);
+    if (! _client->hasStrut()) {
+        if (fixGeometry()) {
+            moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
         }
     }
 
@@ -227,7 +197,7 @@ Frame::~Frame(void)
     woListRemove(this);
     _frame_list.remove(this);
     Workspaces::instance()->remove(this);
-    _wm->removeFromFrameList(this);
+    WindowManager::inst()->removeFromFrameList(this);
     if (_tag_frame == this) {
         _tag_frame = NULL;
     }
@@ -247,8 +217,9 @@ Frame::~Frame(void)
 void
 Frame::iconify(void)
 {
-    if (_iconified)
+    if (_iconified) {
         return;
+    }
     _iconified = true;
 
     unmapWindow();
@@ -261,7 +232,7 @@ Frame::stick(void)
     _client->setSticky(_sticky); // FIXME: FRAME
     _client->stick();
 
-    _sticky = !_sticky;
+    _sticky = ! _sticky;
 
     // make sure it's visible/hidden
     PDecor::setWorkspace(Workspaces::instance()->getActive());
@@ -292,7 +263,7 @@ Frame::handleMotionEvent(XMotionEvent *ev)
 {
     // This is true when we have a title button pressed and then we don't want
     // to be able to drag windows around, therefore we ignore the event
-    if (_button != NULL) {
+    if (_button) {
         return NULL;
     }
 
@@ -314,8 +285,8 @@ Frame::handleMotionEvent(XMotionEvent *ev)
     }
 
     // check motion threshold
-    if ((ae != NULL) && (ae->threshold > 0)) {
-        if (!ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
+    if (ae && (ae->threshold > 0)) {
+        if (! ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
                                              _pointer_x, _pointer_y, ae->threshold)) {
             ae = NULL;
         }
@@ -344,7 +315,7 @@ Frame::handleEnterEvent(XCrossingEvent *ev)
         }
     }
 
-    if (al != NULL) {
+    if (al) {
         ae = ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_ENTER, al);
     }
 
@@ -372,11 +343,11 @@ Frame::handleLeaveEvent(XCrossingEvent *ev)
 ActionEvent*
 Frame::handleMapRequest(XMapRequestEvent *ev)
 {
-    if ((_client == NULL) || (ev->window != _client->getWindow())) {
+    if (! _client || (ev->window != _client->getWindow())) {
         return NULL;
     }
 
-    if (!_sticky && (_workspace != Workspaces::instance()->getActive())) {
+    if (! _sticky && (_workspace != Workspaces::instance()->getActive())) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Ignoring MapRequest, not on current workspace!" << endl;
@@ -411,7 +382,7 @@ Frame::handleUnmapEvent(XUnmapEvent *ev)
 void
 Frame::handleShapeEvent(XAnyEvent *ev)
 {
-    if ((_client == NULL) || (ev->window != _client->getWindow())) {
+    if (! _client || (ev->window != _client->getWindow())) {
         return;
     }
     _client->setShaped(setShape());
@@ -451,13 +422,15 @@ Frame::activateChild (PWinObj *child)
     // setShape uses current active child, so we need to activate the child
     // before setting shape
 #ifdef HAVE_SHAPE
-    if (PScreen::instance ()->hasExtensionShape ())
+    if (PScreen::instance ()->hasExtensionShape ()) {
         _client->setShaped (setShape());
+    }
 #endif // HAVE_SHAPE
 
-    if (_focused)
+    if (_focused) {
         child->giveInputFocus ();
-
+    }
+    
     Workspaces::instance ()->updateClientStackingList (true, true);
 }
 
@@ -494,18 +467,16 @@ Frame::updatedActiveChild(void)
 
 //! @brief
 void
-Frame::getDecorInfo(char *buf, uint size)
+Frame::getDecorInfo(wchar_t *buf, uint size)
 {
     uint width, height;
-    if (_client != NULL) {
+    if (_client) {
         calcSizeInCells(width, height);
     } else {
         width = _gm.width;
         height = _gm.height;
     }
-    snprintf(buf, size, "%d+%d+%d+%d", width, height,
-             _gm.x + Workspaces::instance()->getActiveViewport()->getX(),
-             _gm.y + Workspaces::instance()->getActiveViewport()->getY());
+    swprintf(buf, size, L"%d+%d+%d+%d", width, height, _gm.x, _gm.y);
 }
 
 //! @brief
@@ -524,7 +495,7 @@ Frame::setShaded(StateAction sa)
 int
 Frame::resizeHorzStep(int diff) const
 {
-    if (_client == NULL) {
+    if (! _client) {
         return diff;
     }
 
@@ -564,7 +535,7 @@ Frame::resizeHorzStep(int diff) const
 int
 Frame::resizeVertStep(int diff) const
 {
-    if (_client == NULL) {
+    if (! _client) {
         return diff;
     }
 
@@ -619,13 +590,13 @@ Frame::setId(uint id)
 void
 Frame::getState(Client *cl)
 {
-    if (!cl)
+    if (! cl)
         return;
 
     bool b_client_iconified = cl->isIconified ();
 
     if (_sticky != cl->isSticky())
-        _sticky = !_sticky;
+        _sticky = ! _sticky;
     if (_maximized_horz != cl->isMaximizedHorz())
         setStateMaximized(STATE_TOGGLE, true, false, false);
     if (_maximized_vert != cl->isMaximizedVert())
@@ -646,25 +617,27 @@ Frame::getState(Client *cl)
     if (_fullscreen != cl->isFullscreen())
         setStateFullscreen(STATE_TOGGLE);
 
-    if (_iconified != b_client_iconified)
-    {
-        if (_iconified)
+    if (_iconified != b_client_iconified) {
+        if (_iconified) {
             mapWindow();
-        else
+        } else {
             iconify();
+        }
     }
 
-    if (_skip != cl->getSkip())
+    if (_skip != cl->getSkip()) {
         setSkip(cl->getSkip());
+    }
 }
 
 //! @brief Applies the frame's state on the Client
 void
 Frame::applyState(Client *cl)
 {
-    if (!cl)
+    if (! cl) {
         return;
-
+    }
+    
     cl->setSticky(_sticky);
     cl->setMaximizedHorz(_maximized_horz);
     cl->setMaximizedVert(_maximized_vert);
@@ -677,7 +650,7 @@ Frame::applyState(Client *cl)
     cl->setTitlebar(hasTitlebar());
     // make sure the window has the correct mapped state
     if (_mapped != cl->isMapped()) {
-        if (_mapped == false) {
+        if (! _mapped) {
             cl->unmapWindow();
         } else {
             cl->mapWindow();
@@ -694,7 +667,7 @@ Frame::setSkip(uint skip)
     PDecor::setSkip(skip);
 
     // Propagate changes on client as well
-    if (_client != NULL) {
+    if (_client) {
         _client->setSkip(skip);
     }
 }
@@ -767,27 +740,41 @@ Frame::setupAPGeometry(Client *client, AutoProperty *ap)
     }
 }
 
-//! @brief
+//! @brief Apply autoproperties geometry.
+//! @param gm Geometry to modify.
+//! @param ap_gm Geometry to get values from.
+//! @param mask Geometry mask.
 void
 Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
 {
-    // read size before position so negative position works
-    if (mask&WidthValue)
-        gm.width = ap_gm.width;
-    if (mask&HeightValue)
-        gm.height = ap_gm.height;
-
-    // read position
-    if (mask&(XValue|YValue)) {
-        if (mask&XValue) {
-            gm.x = ap_gm.x;
-            if (mask&XNegative)
-                gm.x += _wm->getScreen()->getWidth() - gm.width;
+    // Read size before position so negative position works, if size is
+    // < 1 consider it to be full screen size.
+    if (mask&WidthValue) {
+        if (ap_gm.width < 1) {
+            gm.width = _scr->getWidth();
+        } else {
+            gm.width = ap_gm.width;
         }
-        if (mask&YValue) {
-            gm.y = ap_gm.y;
-            if (mask&YNegative)
-                gm.y += _wm->getScreen()->getHeight() - gm.height;
+    }
+    
+    if (mask&HeightValue) {
+        if (ap_gm.height < 1) {
+        } else {
+            gm.height = ap_gm.height;
+        }
+    }
+
+    // Read position
+    if (mask&XValue) {
+        gm.x = ap_gm.x;
+        if (mask&XNegative) {
+            gm.x += _scr->getWidth() - gm.width;
+        }
+    }
+    if (mask&YValue) {
+        gm.y = ap_gm.y;
+        if (mask&YNegative) {
+            gm.y += _scr->getHeight() - gm.height;
         }
     }
 }
@@ -822,6 +809,18 @@ Frame::returnFrameID(uint id)
     _frameid_list.insert(it, id);
 }
 
+/**
+ * Return decor name matching clients property, defaults to
+ * PDecor::DEFAULT_DECOR_NAME
+ */
+std::string
+Frame::getClientDecorName(Client *client)
+{
+  DecorProperty *prop = WindowManager::inst()->getAutoProperties()->findDecorProperty(client->getClassHint());
+
+  return prop ? prop->getName() : PDecor::DEFAULT_DECOR_NAME;
+}
+
 //! @brief Resets Frame IDs.
 void
 Frame::resetFrameIDs(void)
@@ -845,7 +844,7 @@ Frame::detachClient(Client *client)
         removeChild(client);
 
         client->move(_gm.x, _gm.y + borderTop());
-        Frame *frame = new Frame(_wm, client, NULL);
+        Frame *frame = new Frame(client, NULL);
 
         client->setParent(frame);
         client->setWorkspace(Workspaces::instance()->getActive());
@@ -889,24 +888,26 @@ Frame::fixGeometry(void)
 void
 Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: rewrite
 {
-    if (client == NULL)
+    if (! client) {
         return;
+    }
 
     int o_x, o_y;
     o_x = ev ? ev->x_root : 0;
     o_y = ev ? ev->y_root : 0;
 
-    string name("Grouping ");
+    wstring name(L"Grouping ");
     if (client->getTitle()->getVisible().size() > 0) {
         name += client->getTitle()->getVisible();
     } else {
-        name += "No Name";
+        name += L"No Name";
     }
 
     bool status = _scr->grabPointer(_scr->getRoot(),
                                     ButtonReleaseMask|PointerMotionMask, None);
-    if (status != true)
+    if (status != true) {
         return;
+    }
 
     StatusWindow *sw = StatusWindow::instance();
 
@@ -936,7 +937,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
             Client *search = NULL;
 
             // only group if we have grouping turned on
-            if (_wm->isAllowGrouping()) {
+            if (WindowManager::inst()->isAllowGrouping()) {
                 int x, y;
                 Window win;
 
@@ -950,7 +951,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
             // if we found a client, and it's not in the current frame and
             // it has a "normal" ( make configurable? ) layer we group
-            if ((search != NULL) && (search->getParent() != NULL) &&
+            if (search && search->getParent() &&
                     (search->getParent() != this) &&
                     (search->getLayer() > LAYER_BELOW) &&
                     (search->getLayer() < LAYER_ONTOP)) {
@@ -963,7 +964,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
                 Frame *frame = static_cast<Frame*>(search->getParent());
                 frame->addChild(client);
-                if (behind == false) {
+                if (! behind) {
                     frame->activateChild(client);
                     frame->giveInputFocus();
                 }
@@ -978,7 +979,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
                 client->move(e.xmotion.x_root, e.xmotion.y_root);
 
-                Frame *frame = new Frame(_wm, client, NULL);
+                Frame *frame = new Frame(client, NULL);
                 client->setParent(frame);
 
                 // make sure the client ends up on the current workspace
@@ -998,16 +999,19 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 void
 Frame::doResize(XMotionEvent *ev)
 {
-    if (!ev)
+    if (! ev) {
         return;
-
+    }
+    
     // figure out which part of the window we are in
     bool left = false, top = false;
-    if (ev->x < signed(_gm.width / 2))
+    if (ev->x < signed(_gm.width / 2)) {
         left = true;
-    if (ev->y < signed(_gm.height / 2))
+    }
+    if (ev->y < signed(_gm.height / 2)) {
         top = true;
-
+    }
+    
     doResize(left, true, top, true);
 }
 
@@ -1057,13 +1061,14 @@ Frame::doResize(BorderPosition pos)
 void
 Frame::doResize(bool left, bool x, bool top, bool y)
 {
-    if ((_client->allowResize() == false) || isShaded()) {
+    if (! _client->allowResize() || isShaded()) {
         return;
     }
 
-    if (!_scr->grabPointer(_scr->getRoot(), ButtonMotionMask|ButtonReleaseMask,
-                           ScreenResources::instance()->getCursor(ScreenResources::CURSOR_RESIZE)))
+    if (! _scr->grabPointer(_scr->getRoot(), ButtonMotionMask|ButtonReleaseMask,
+                           ScreenResources::instance()->getCursor(ScreenResources::CURSOR_RESIZE))) {
         return;
+    }
 
     setShaded(STATE_UNSET); // make sure the frame isn't shaded
 
@@ -1085,8 +1090,8 @@ Frame::doResize(bool left, bool x, bool top, bool y)
     int pointer_x = _gm.x, pointer_y = _gm.y;
     _scr->getMousePosition(pointer_x, pointer_y);
 
-    char buf[128];
-    getDecorInfo(buf, sizeof(buf)/sizeof(char));
+    wchar_t buf[128];
+    getDecorInfo(buf, 128);
 
     StatusWindow *sw = StatusWindow::instance();
     if (Config::instance()->isShowStatusWindow()) {
@@ -1095,7 +1100,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
         sw->draw(buf);
     }
 
-    bool outline = (Config::instance()->getOpaqueResize() == false);
+    bool outline = ! Config::instance()->getOpaqueResize();
 
     // grab server, we don't want invert traces
     if (outline) {
@@ -1124,16 +1129,15 @@ Frame::doResize(bool left, bool x, bool top, bool y)
 
             recalcResizeDrag(new_x, new_y, left, top);
 
-            getDecorInfo(buf, sizeof(buf)/sizeof(char));
+            getDecorInfo(buf, 128);
             if (Config::instance()->isShowStatusWindow()) {
                 sw->draw(buf, true);
             }
 
             // only updated when needed when in opaque mode
-            if (outline == false) {
+            if (! outline) {
                 if ((old_width != _gm.width) || (old_height != _gm.height)) {
-                    resize(_gm.width, _gm.height);
-                    move(_gm.x, _gm.y);
+                    moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
                 }
                 old_width = _gm.width;
                 old_height = _gm.height;
@@ -1161,11 +1165,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
     }
 
     if (outline) {
-        resize(_gm.width, _gm.height);
-        move(_gm.x, _gm.y);
-    }
-
-    if (outline) {
+        moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
         _scr->ungrabServer(true);
     }
 }
@@ -1294,9 +1294,10 @@ Frame::moveToEdge(OrientationType ori)
 void
 Frame::updateInactiveChildInfo(void)
 {
-    if (_client == NULL)
+    if (! _client) {
         return;
-
+    }
+    
     list<PWinObj*>::iterator it(_child_list.begin());
     for (; it != _child_list.end(); ++it) {
         if (*it != _client) {
@@ -1326,8 +1327,8 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
     // make sure the two states are in sync if toggling
     if ((horz == vert) && (sa == STATE_TOGGLE)) {
         if (_maximized_horz != _maximized_vert) {
-            horz = !_maximized_horz;
-            vert = !_maximized_vert;
+            horz = ! _maximized_horz;
+            vert = ! _maximized_vert;
         }
     }
 
@@ -1352,10 +1353,10 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
     if (horz && (fill || _client->allowMaximizeHorz())) {
         // maximize
         if ((sa == STATE_SET) ||
-                ((sa == STATE_TOGGLE) && (_maximized_horz == false))) {
+                ((sa == STATE_TOGGLE) && ! _maximized_horz)) {
             uint h_decor = _gm.width - getChildWidth();
 
-            if (fill == false) {
+            if (! fill) {
                 _old_gm.x = _gm.x;
                 _old_gm.width = _gm.width;
             }
@@ -1369,23 +1370,23 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
             }
             // demaximize
         } else if ((sa == STATE_UNSET) ||
-                   ((sa == STATE_TOGGLE) && (_maximized_horz == true))) {
+                   ((sa == STATE_TOGGLE) && _maximized_horz)) {
             _gm.x = _old_gm.x;
             _gm.width = _old_gm.width;
         }
 
         // we unset the maximized state if we use maxfill
-        _maximized_horz = fill ? false : !_maximized_horz;
+        _maximized_horz = fill ? false : ! _maximized_horz;
         _client->setMaximizedHorz(_maximized_horz);
     }
 
     if (vert && (fill || _client->allowMaximizeVert())) {
         // maximize
         if ((sa == STATE_SET) ||
-                ((sa == STATE_TOGGLE) && (_maximized_vert == false))) {
+                ((sa == STATE_TOGGLE) && ! _maximized_vert)) {
             uint v_decor = _gm.height - getChildHeight();
 
-            if (fill == false) {
+            if (! fill) {
                 _old_gm.y = _gm.y;
                 _old_gm.height = _gm.height;
             }
@@ -1399,30 +1400,29 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
             }
             // demaximize
         } else if ((sa == STATE_UNSET) ||
-                   ((sa == STATE_TOGGLE) && (_maximized_vert == true))) {
+                   ((sa == STATE_TOGGLE) && _maximized_vert)) {
             _gm.y = _old_gm.y;
             _gm.height = _old_gm.height;
         }
 
         // we unset the maximized state if we use maxfill
-        _maximized_vert = fill ? false : !_maximized_vert;
+        _maximized_vert = fill ? false : ! _maximized_vert;
         _client->setMaximizedVert(_maximized_vert);
     }
 
     fixGeometry(); // harbour already considered
     downSize(true, true); // keep x and keep y ( make conform to inc size )
 
-    resize(_gm.width, _gm.height);
-    move(_gm.x, _gm.y);
+    moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
     _client->updateEwmhStates();
 }
 
-//! @brief
+//! @brief Set fullscreen state
 void
 Frame::setStateFullscreen(StateAction sa)
 {
-    if (ActionUtil::needToggle(sa, _fullscreen) == false) {
+    if (! ActionUtil::needToggle(sa, _fullscreen)) {
         return;
     }
 
@@ -1454,8 +1454,7 @@ Frame::setStateFullscreen(StateAction sa)
     _fullscreen = !_fullscreen;
     _client->setFullscreen(_fullscreen);
 
-    move(_gm.x, _gm.y);
-    resize(_gm.width, _gm.height);
+    moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
     _client->setConfigureRequestLock(lock);
     _client->configureRequestSend();
@@ -1467,7 +1466,7 @@ Frame::setStateFullscreen(StateAction sa)
 void
 Frame::setStateSticky(StateAction sa)
 {
-    if (ActionUtil::needToggle(sa, _sticky) == true) {
+    if (ActionUtil::needToggle(sa, _sticky)) {
         stick();
     }
 }
@@ -1476,7 +1475,7 @@ Frame::setStateSticky(StateAction sa)
 void
 Frame::setStateAlwaysOnTop(StateAction sa)
 {
-    if (ActionUtil::needToggle(sa, _layer == LAYER_ONTOP) == false) {
+    if (! ActionUtil::needToggle(sa, _layer == LAYER_ONTOP)) {
         return;
     }
 
@@ -1490,7 +1489,7 @@ Frame::setStateAlwaysOnTop(StateAction sa)
 void
 Frame::setStateAlwaysBelow(StateAction sa)
 {
-    if (ActionUtil::needToggle(sa, _layer == LAYER_BELOW) == false) {
+    if (! ActionUtil::needToggle(sa, _layer == LAYER_BELOW)) {
         return;
     }
 
@@ -1543,7 +1542,7 @@ Frame::setStateDecorTitlebar(StateAction sa)
 void
 Frame::setStateIconified(StateAction sa)
 {
-    if (ActionUtil::needToggle(sa, _iconified) == false) {
+    if (! ActionUtil::needToggle(sa, _iconified)) {
         return;
     }
 
@@ -1571,7 +1570,7 @@ Frame::setStateTagged(StateAction sa, bool behind)
 void
 Frame::setStateSkip(StateAction sa, uint skip)
 {
-    if (ActionUtil::needToggle(sa, _skip&skip) == false) {
+    if (! ActionUtil::needToggle(sa, _skip&skip)) {
         return;
     }
 
@@ -1586,17 +1585,17 @@ Frame::setStateSkip(StateAction sa, uint skip)
 
 //! @brief Sets client title
 void
-Frame::setStateTitle(StateAction sa, Client *client, const std::string &title)
+Frame::setStateTitle(StateAction sa, Client *client, const std::wstring &title)
 {
     if (sa == STATE_SET) {
         client->getTitle()->setUser(title);
 
     } else if (sa == STATE_UNSET) {
-        client->getTitle()->setUser("");
+        client->getTitle()->setUser(L"");
         client->getXClientName();
     } else {
         if (client->getTitle()->isUserSet()) {
-            client->getTitle()->setUser("");
+            client->getTitle()->setUser(L"");
         } else {
             client->getTitle()->setUser(title);
         }
@@ -1605,7 +1604,7 @@ Frame::setStateTitle(StateAction sa, Client *client, const std::string &title)
     // Set PEKWM_TITLE atom to preserve title on client between sessions.
     AtomUtil::setString(client->getWindow(),
                         PekwmAtoms::instance()->getAtom(PEKWM_TITLE),
-                        client->getTitle()->getUser());
+                        Util::to_mb_str(client->getTitle()->getUser()));
     
 
     renderTitle();
@@ -1617,7 +1616,7 @@ Frame::setStateTitle(StateAction sa, Client *client, const std::string &title)
 void
 Frame::setStateMarked(StateAction sa, Client *client)
 {
-    if (!client || !ActionUtil::needToggle(sa, client->isMarked())) {
+    if (! client || ! ActionUtil::needToggle(sa, client->isMarked())) {
         return;
     }
 
@@ -1640,7 +1639,7 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
 
     list<Frame*>::iterator it = _frame_list.begin();
     for (; it != _frame_list.end(); ++it) {
-        if ((*it)->isMapped() == false) {
+        if (! (*it)->isMapped()) {
             continue;
         }
 
@@ -1653,16 +1652,16 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
 
         // update max borders when other frame border lies between
         // this border and prior max border (originally screen/head edge)
-        if ((r >= max_x) && (r <= _gm.x) && !((y >= f_b) || (b <= _gm.y))) {
+        if ((r >= max_x) && (r <= _gm.x) && ! ((y >= f_b) || (b <= _gm.y))) {
             max_x = r;
         }
-        if ((x <= max_r) && (x >= f_r) && !((y >= f_b) || (b <= _gm.y))) {
+        if ((x <= max_r) && (x >= f_r) && ! ((y >= f_b) || (b <= _gm.y))) {
             max_r = x;
         }
-        if ((b >= max_y) && (b <= _gm.y) && !((x >= f_r) || (r <= _gm.x))) {
+        if ((b >= max_y) && (b <= _gm.y) && ! ((x >= f_r) || (r <= _gm.x))) {
             max_y = b;
         }
-        if ((y <= max_b) && (y >= f_b) && !((x >= f_r) || (r <= _gm.x))) {
+        if ((y <= max_b) && (y >= f_b) && ! ((x >= f_r) || (r <= _gm.x))) {
             max_b = y;
         }
     }
@@ -1696,8 +1695,7 @@ Frame::growDirection(uint direction)
 
     downSize((direction != DIRECTION_LEFT), (direction != DIRECTION_UP));
 
-    move(_gm.x, _gm.y);
-    resize(_gm.width, _gm.height);
+    moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 }
 
 //! @brief Closes the frame and all clients in it
@@ -1721,24 +1719,27 @@ Frame::readAutoprops(uint type)
     _class_hint->title = _client->getTitle()->getReal();
     AutoProperty *data =
         AutoProperties::instance()->findAutoProperty(_class_hint, _workspace, type);
-    _class_hint->title = "";
+    _class_hint->title = L"";
 
-    if (data == NULL)
+    if (! data) {
         return;
+    }
 
     // Set the correct group of the window
     _class_hint->group = data->group_name;
 
-    if ((_class_hint == _client->getClassHint()) &&
+    if (_class_hint == _client->getClassHint() &&
             (_client->getTransientWindow() &&
-             !data->isApplyOn(APPLY_ON_TRANSIENT))) {
+             ! data->isApplyOn(APPLY_ON_TRANSIENT))) {
         return;
     }
 
-    if (data->isMask(AP_STICKY) && (_sticky != data->sticky))
+    if (data->isMask(AP_STICKY) && _sticky != data->sticky) {
         stick();
-    if (data->isMask(AP_SHADED) && (isShaded() != data->shaded))
+    }
+    if (data->isMask(AP_SHADED) && (isShaded() != data->shaded)) {
         setShaded(STATE_UNSET);
+    }
     if (data->isMask(AP_MAXIMIZED_HORIZONTAL) &&
             (_maximized_horz != data->maximized_horizontal)) {
         setStateMaximized(STATE_TOGGLE, true, false, false);
@@ -1752,8 +1753,11 @@ Frame::readAutoprops(uint type)
     }
 
     if (data->isMask(AP_ICONIFIED) && (_iconified != data->iconified)) {
-        if (_iconified) mapWindow();
-        else iconify();
+        if (_iconified) {
+            mapWindow();
+        } else {
+            iconify();
+        }
     }
     if (data->isMask(AP_WORKSPACE)) {
         // In order to avoid eternal recursion, the workspace is only set here
@@ -1766,8 +1770,9 @@ Frame::readAutoprops(uint type)
         }
     }
 
-    if (data->isMask(AP_SHADED) && (isShaded() != data->shaded))
+    if (data->isMask(AP_SHADED) && (isShaded() != data->shaded)) {
         setShaded(STATE_TOGGLE);
+    }
     if (data->isMask(AP_LAYER) && (data->layer <= LAYER_MENU)) {
         _client->setLayer(data->layer);
         raise(); // restack the frame
@@ -1776,15 +1781,16 @@ Frame::readAutoprops(uint type)
     if (data->isMask(AP_FRAME_GEOMETRY|AP_CLIENT_GEOMETRY)) {
         setupAPGeometry(_client, data);
 
-        // apply changes
-        move(_gm.x, _gm.y);
-        resize(_gm.width, _gm.height);
+        // Apply changes
+        moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
     }
 
-    if (data->isMask(AP_BORDER) && (hasBorder() != data->border))
+    if (data->isMask(AP_BORDER) && (hasBorder() != data->border)) {
         setStateDecorBorder(STATE_TOGGLE);
-    if (data->isMask(AP_TITLEBAR) && (hasTitlebar() != data->titlebar))
+    }
+    if (data->isMask(AP_TITLEBAR) && (hasTitlebar() != data->titlebar)) {
         setStateDecorTitlebar(STATE_TOGGLE);
+    }
 
     if (data->isMask(AP_SKIP)) {
         _client->setSkip(data->skip);
@@ -1876,7 +1882,7 @@ Frame::downSize(bool keep_x, bool keep_y)
                   : (size_hint->flags&PMinSize) ? size_hint->min_width : 0;
 
         _gm.width -= (getChildWidth() - b_x) % size_hint->width_inc;
-        if (keep_x == false) {
+        if (! keep_x) {
             _gm.x = o_r - _gm.width;
         }
     }
@@ -1889,7 +1895,7 @@ Frame::downSize(bool keep_x, bool keep_y)
                   : (size_hint->flags&PMinSize) ? size_hint->min_height : 0;
 
         _gm.height -= (getChildHeight() - b_y) % size_hint->height_inc;
-        if (keep_y == false) {
+        if (! keep_y) {
             _gm.y = o_b - _gm.height;
         }
     }
@@ -1907,7 +1913,7 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
     }
 
     // size before position, as we rely on size when gravitating
-    if (client->isCfgDeny(CFG_DENY_SIZE) == false) {
+    if (! client->isCfgDeny(CFG_DENY_SIZE)) {
         if ((ev->value_mask&CWWidth) || (ev->value_mask&CWHeight)) {
             resizeChild(ev->width, ev->height);
 #ifdef HAVE_SHAPE
@@ -1916,7 +1922,7 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
         }
     }
 
-    if (client->isCfgDeny(CFG_DENY_POSITION) == false) {
+    if (! client->isCfgDeny(CFG_DENY_POSITION)) {
         if ((ev->value_mask&CWX) || (ev->value_mask&CWY)) {
             calcGravityPosition(_client->getXSizeHints()->win_gravity,
                                 ev->x, ev->y, _gm.x, _gm.y);
@@ -1925,7 +1931,7 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
     }
 
     // update the stacking
-    if (client->isCfgDeny(CFG_DENY_STACKING) == false) {
+    if (! client->isCfgDeny(CFG_DENY_STACKING)) {
         if (ev->value_mask&CWStackMode) {
             if (ev->value_mask&CWSibling) {
                 switch(ev->detail) {
@@ -1994,67 +2000,73 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
                 setStateSticky(sa);
             }
             if (IS_STATE(STATE_MAXIMIZED_HORZ)
-                    && !client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
                 setStateMaximized(sa, true, false, false);
             }
             if (IS_STATE(STATE_MAXIMIZED_VERT)
-                    && !client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
                 setStateMaximized(sa, false, true, false);
             }
             if (IS_STATE(STATE_SHADED)) {
                 setShaded(sa);
             }
             if (IS_STATE(STATE_HIDDEN)
-                    && !client->isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
                 setStateIconified(sa);
             }
             if (IS_STATE(STATE_FULLSCREEN)
-                    && !client->isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
                 setStateFullscreen(sa);
             }
             if (IS_STATE(STATE_ABOVE)
-                    && !client->isCfgDeny(CFG_DENY_STATE_ABOVE)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_ABOVE)) {
                 setStateAlwaysOnTop(sa);
             }
             if (IS_STATE(STATE_BELOW)
-                    && !client->isCfgDeny(CFG_DENY_STATE_BELOW)) {
+                    && ! client->isCfgDeny(CFG_DENY_STATE_BELOW)) {
                 setStateAlwaysBelow(sa);
             }
         }
 
         if (IS_STATE(STATE_SKIP_TASKBAR)) {
-            client->setStateSkipTaskbar(sa);
+            client->setStateSkip(sa, SKIP_TASKBAR);
         }
         if (IS_STATE(STATE_SKIP_PAGER)) {
-            client->setStateSkipPager(sa);
+            client->setStateSkip(sa, SKIP_PAGER);
         }
 
         client->updateEwmhStates();
 
     } else if (ev->message_type == ewmh->getAtom(NET_ACTIVE_WINDOW)) {
-        if (!client->isCfgDeny(CFG_DENY_ACTIVE_WINDOW)) {
+        if (! client->isCfgDeny(CFG_DENY_ACTIVE_WINDOW)) {
             // Active child if it's not the active child
-            if (client != _client)
+            if (client != _client) {
                 activateChild(client);
+	        }
+
             // If we aren't mapped we check if we make sure we're on the right
             // workspace and then map the window.
-            if (!_mapped) {
+            if (! _mapped) {
                 if (_workspace != Workspaces::instance()->getActive()) {
                     Workspaces::instance()->setWorkspace(_workspace, false);
                 }
                 mapWindow();
             }
+            // Seems as if raising the window is implied in activating it
+            raise();
             giveInputFocus();
         }
     } else if (ev->message_type == ewmh->getAtom(NET_CLOSE_WINDOW)) {
         client->close();
     } else if (ev->message_type == ewmh->getAtom(NET_WM_DESKTOP)) {
-        if (client == _client)
+        if (client == _client) {
             setWorkspace(ev->data.l[0]);
+        }
     } else if (ev->message_type == IcccmAtoms::instance()->getAtom(WM_CHANGE_STATE) &&
                (ev->format == 32) && (ev->data.l[0] == IconicState)) {
-        if (client == _client)
+        if (client == _client) {
             iconify();
+        }
     }
 #undef IS_STATE
 }
@@ -2075,16 +2087,14 @@ Frame::handlePropertyChange(XPropertyEvent *ev, Client *client)
                 if (workspace != signed(_workspace))
                     setWorkspace(workspace);
             }
-
         }
     } else if (ev->atom == ewmh->getAtom(NET_WM_STRUT)) {
         client->getStrutHint();
     } else if (ev->atom == ewmh->getAtom(NET_WM_NAME)) {
-        // FIXME: UTF8 support so we can handle the hint
         client->getXClientName();
         renderTitle();
     } else if (ev->atom == XA_WM_NAME) {
-//	if (!m_has_extended_net_name)
+//	if (! m_has_extended_net_name)
         client->getXClientName();
         renderTitle();
     } else if (ev->atom == XA_WM_NORMAL_HINTS) {
