@@ -20,7 +20,7 @@
 
 //! @brief HarbourMenu constructor
 HarbourMenu::HarbourMenu(PScreen *scr, Theme *theme, Harbour *harbour) :
-        PMenu(scr->getDpy(), theme, "Harbour", "HARBOUR"),
+        PMenu(scr->getDpy(), theme, L"Harbour", "HARBOUR"),
         _harbour(harbour),
         _dockapp(NULL)
 {
@@ -48,7 +48,7 @@ HarbourMenu::~HarbourMenu(void)
 void
 HarbourMenu::handleItemExec(PMenu::Item *item)
 {
-    if ((item == NULL) || (_dockapp == NULL)) {
+    if (! item || ! _dockapp) {
         return;
     }
 
