@@ -1,17 +1,33 @@
 //
 // PDecor.cc for pekwm
-// Copyright (C) 2004-2006 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2004-2008 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id: PDecor.cc 124 2007-01-31 22:07:37Z me@pekdon.net $
-//
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
+#include <cassert>
+#include <cstdio>
+#include <cstdlib>
+#include <functional>
+#include <iostream>
+#include <cwchar>
+
+extern "C" {
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#ifdef HAVE_SHAPE
+#include <X11/extensions/shape.h>
+#endif // HAVE_SHAPE
+}
 
 #include "Config.hh"
 #include "PWinObj.hh"
+#include "PFont.hh"
 #include "PDecor.hh"
 #include "PScreen.hh"
 #include "PTexture.hh"
@@ -23,44 +39,28 @@
 #include "Theme.hh"
 #include "PixmapHandler.hh"
 #include "Workspaces.hh"
-#include "Viewport.hh"
 
-#include <functional>
-#include <cassert>
-#include <cstdio>
-#include <cstdlib>
-
-extern "C" {
-#ifdef HAVE_SHAPE
-#include <X11/Xutil.h>
-#include <X11/extensions/shape.h>
-#endif // HAVE_SHAPE
-}
-
-#ifdef DEBUG
-#include <iostream>
 using std::cerr;
 using std::endl;
-#endif // DEBUG
-
 using std::find;
 using std::list;
-using std::vector;
-using std::string;
 using std::map;
 using std::mem_fun;
+using std::string;
+using std::vector;
 
 // PDecor::Button
 
 //! @brief PDecor::Button constructor
-PDecor::Button::Button(Display *dpy, PWinObj *parent, Theme::PDecorButtonData *data) : PWinObj(dpy),
-        _data(data), _state(BUTTON_STATE_UNFOCUSED),
-        _left(_data->isLeft())
+PDecor::Button::Button(Display *dpy, PWinObj *parent, Theme::PDecorButtonData *data, uint width, uint height)
+  : PWinObj(dpy),
+    _data(data), _state(BUTTON_STATE_UNFOCUSED),
+    _left(_data->isLeft())
 {
     _parent = parent;
 
-    _gm.width = _data->getWidth();
-    _gm.height = _data->getHeight();
+    _gm.width = width;
+    _gm.height = height;
 
     XSetWindowAttributes attr;
     attr.override_redirect = True;
@@ -110,6 +110,7 @@ PDecor::Button::setState(ButtonState state)
 
     _data->getTexture(state)->render(_bg, 0, 0, _gm.width, _gm.height);
 
+#ifdef HAVE_SHAPE
     // Get shape mask
     bool need_free;
     Pixmap shape = _data->getTexture(state)->getMask(0, 0, need_free);
@@ -123,7 +124,7 @@ PDecor::Button::setState(ButtonState state)
         XShapeCombineRectangles(_dpy, _window, ShapeBounding,
                                 0, 0, &rect, 1, ShapeSet, YXBanded);
     }
-
+#endif
     clear();
 }
 
@@ -131,21 +132,21 @@ PDecor::Button::setState(ButtonState state)
 void
 PDecor::TitleItem::updateVisible(void) {
     // Start with empty string
-    _visible = "";
+    _visible = L"";
 
     // Add client info to title
     if ((_info != 0)
         && ((_info != INFO_ID) || Config::instance()->isShowClientID())) {
-        _visible.append("[");
+        _visible.append(L"[");
 
         if (infoIs(INFO_ID) && Config::instance()->isShowClientID()) {
-            _visible.append(Util::to_string<uint>(_id));
+            _visible.append(Util::to_wide_str(Util::to_string<uint>(_id)));
         }
         if (infoIs(INFO_MARKED)) {
-            _visible.append("M");
+            _visible.append(L"M");
         }
 
-        _visible.append("] ");
+        _visible.append(L"] ");
     }
 
     // Add title
@@ -159,9 +160,9 @@ PDecor::TitleItem::updateVisible(void) {
 
     // Add client number to title
     if (_count > 0) {
-        _visible.append(Config::instance()->getClientUniqueNamePre());
-        _visible.append(Util::to_string(_count));
-        _visible.append(Config::instance()->getClientUniqueNamePost());
+      _visible.append(Util::to_wide_str(Config::instance()->getClientUniqueNamePre()));
+      _visible.append(Util::to_wide_str(Util::to_string(_count)));
+      _visible.append(Util::to_wide_str(Config::instance()->getClientUniqueNamePost()));
     }
 }
 
@@ -200,7 +201,7 @@ PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
     // we be reset in loadDecor later on, inlines using the _data used before
     // loadDecor needs this though
     _data = _theme->getPDecorData(_decor_name);
-    if (_data == NULL) {
+    if (! _data) {
         _data = _theme->getPDecorData(DEFAULT_DECOR_NAME);
     }
 
@@ -253,8 +254,10 @@ PDecor::~PDecor(void)
         }
     }
 
-    // make things look smoother, buttons will be noticed as deleted otherwise
-    unmapWindow();
+    // Make things look smoother, buttons will be noticed as deleted
+    // otherwise. Using X call directly to avoid re-drawing and other
+    // special features not required when removing the window.
+    XUnmapWindow(_dpy, _window);
 
     // free buttons
     unloadDecor();
@@ -283,7 +286,7 @@ PDecor::~PDecor(void)
 void
 PDecor::mapWindow(void)
 {
-    if (_mapped != true) {
+    if (! _mapped) {
         PWinObj::mapWindow();
         for_each(_child_list.begin(), _child_list.end(),
                  mem_fun(&PWinObj::mapWindow));
@@ -294,9 +297,10 @@ PDecor::mapWindow(void)
 void
 PDecor::mapWindowRaised(void)
 {
-    if (_mapped == true) {
+    if (_mapped) {
         return;
     }
+    
     mapWindow();
 
     raise(); // XMapRaised wouldn't preserver layers
@@ -306,8 +310,8 @@ PDecor::mapWindowRaised(void)
 void
 PDecor::unmapWindow(void)
 {
-    if (_mapped == true) {
-        if (_iconified == true) {
+    if (_mapped) {
+        if (_iconified) {
             for_each(_child_list.begin(), _child_list.end(),
                      mem_fun(&PWinObj::iconify));
         } else {
@@ -320,52 +324,35 @@ PDecor::unmapWindow(void)
 
 //! @brief Moves the decor
 void
-PDecor::move(int x, int y, bool do_virtual)
+PDecor::move(int x, int y)
 {
     // update real position
     PWinObj::move(x, y);
-    if ((_child != NULL) && (_decor_cfg_child_move_overloaded)) {
+    if (_child && (_decor_cfg_child_move_overloaded)) {
         _child->move(x + borderLeft(), y + borderTop() + getTitleHeight());
     }
-
-    // update virtual position
-    if (do_virtual == true) {
-        moveVirtual(x + Workspaces::instance()->getActiveViewport()->getX(),
-                    y + Workspaces::instance()->getActiveViewport()->getY());
-    }
-}
-
-//! @brief Virtually moves the decor, and all children if any
-void
-PDecor::moveVirtual(int x, int y)
-{
-    PWinObj::moveVirtual(x, y);
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
-        (*it)->moveVirtual(x + borderLeft(), y + borderTop() + getTitleHeight());
-    }
 }
 
 //! @brief Resizes the decor, and active child if any
 void
 PDecor::resize(uint width, uint height)
 {
-    // if shaded, don't resize to the size specified just update width
+    // If shaded, don't resize to the size specified just update width
     // and set _real_height to height
     if (_shaded) {
         _real_height = height;
 
         height = getTitleHeight();
         // shading in non full width title mode will make border go away
-        if (!_data->getTitleWidthMin())
+        if (! _data->getTitleWidthMin())
             height += borderTop() + borderBottom();
     }
 
     PWinObj::resize(width, height);
 
-    // do before placing and shaping the rest as shaping depends on the
-    // child window
-    if (_child != NULL) {
+    // Update size before moving and shaping the rest as shaping
+    // depends on the child window
+    if (_child) {
         _child->resize(getChildWidth(), getChildHeight());
     }
 
@@ -376,6 +363,53 @@ PDecor::resize(uint width, uint height)
     // render title and border
     _dirty_resized = true;
 
+    // Set and apply shape on window, all parts of the border can now
+    // be shaped.
+    setBorderShape();
+    applyBorderShape();
+
+    renderTitle();
+    renderBorder();
+
+    _dirty_resized = false;
+}
+
+//! @brief Move and resize window.
+void
+PDecor::moveResize(int x, int y, uint width, uint height)
+{
+    // If shaded, don't resize to the size specified just update width
+    // and set _real_height to height
+    if (_shaded) {
+        _real_height = height;
+
+        height = getTitleHeight();
+        // shading in non full width title mode will make border go away
+        if (! _data->getTitleWidthMin()) {
+            height += borderTop() + borderBottom();
+        }
+    }
+
+    PWinObj::moveResize(x, y, width, height);
+
+    // Update size before moving and shaping the rest as shaping
+    // depends on the child window
+    if (_child) {
+        _child->moveResize(x + borderLeft(), y + borderTop() + getTitleHeight(),
+                          getChildWidth(), getChildHeight());
+    }
+
+    // Place and resize title and border
+    resizeTitle();
+    placeBorder();
+
+    // render title and border
+    _dirty_resized = true;
+
+    // Apply shape on window, all parts of the border can now be shaped.
+    setBorderShape();
+    applyBorderShape();
+
     renderTitle();
     renderBorder();
 
@@ -449,11 +483,11 @@ PDecor::setWorkspace(uint workspace)
         (*it)->setWorkspace (workspace);
     }
 
-    if (!_mapped && !_iconified) {
+    if (! _mapped && ! _iconified) {
         if (_sticky || (_workspace == Workspaces::instance()->getActive())) {
             mapWindow();
         }
-    } else if (!_sticky && (_workspace != Workspaces::instance()->getActive())) {
+    } else if (! _sticky && (_workspace != Workspaces::instance()->getActive())) {
         unmapWindow();
     }
 }
@@ -462,8 +496,7 @@ PDecor::setWorkspace(uint workspace)
 bool
 PDecor::giveInputFocus(void)
 {
-    if ((_mapped == true) && (_child != NULL) &&
-            (Workspaces::instance()->getActiveViewport()->isInside(this))) {
+    if (_mapped && _child) {
         return _child->giveInputFocus();
 
     } else {
@@ -499,7 +532,7 @@ PDecor::handleButtonPress(XButtonEvent *ev)
         ae = _button->findAction(ev);
 
         // if the button is used for resizing, we don't want to wait for release
-        if ((ae != NULL) && ae->isOnlyAction(ACTION_RESIZE)) {
+        if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
             _button->setState(_focused ? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
             _button = NULL;
         } else {
@@ -514,7 +547,7 @@ PDecor::handleButtonPress(XButtonEvent *ev)
         _pointer_y = ev->y_root;
 
         // allow us to get clicks from anywhere on the window.
-        if (_decor_cfg_bpr_replay_pointer == true) {
+        if (_decor_cfg_bpr_replay_pointer) {
             XAllowEvents(_dpy, ReplayPointer, CurrentTime);
         }
 
@@ -572,7 +605,7 @@ PDecor::handleButtonRelease(XButtonEvent *ev)
             ae = _button->findAction(ev);
 
             // this is a little hack, resizing isn't wanted on both press and release
-            if ((ae != NULL) && ae->isOnlyAction(ACTION_RESIZE)) {
+            if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
                 ae = NULL;
             }
         }
@@ -582,7 +615,7 @@ PDecor::handleButtonRelease(XButtonEvent *ev)
     } else {
 
         // allow us to get clicks from anywhere on the window.
-        if (_decor_cfg_bpr_replay_pointer == true) {
+        if (_decor_cfg_bpr_replay_pointer) {
             XAllowEvents(_dpy, ReplayPointer, CurrentTime);
         }
 
@@ -727,7 +760,7 @@ PDecor::loadDecor(void)
 
     // Get decordata with name.
     _data = _theme->getPDecorData(_decor_name);
-    if (_data == NULL) {
+    if (! _data) {
         _data = _theme->getPDecorData(DEFAULT_DECOR_NAME);
     }
     assert(_data);
@@ -735,7 +768,10 @@ PDecor::loadDecor(void)
     // Load decor.
     list<Theme::PDecorButtonData*>::iterator b_it(_data->buttonBegin());
     for (; b_it != _data->buttonEnd(); ++b_it) {
-        _button_list.push_back(new PDecor::Button(_dpy, &_title_wo, *b_it));
+        uint width = std::max(static_cast<uint>(1), (*b_it)->getWidth() ? (*b_it)->getWidth() : getTitleHeight());
+        uint height = std::max(static_cast<uint>(1), (*b_it)->getHeight() ? (*b_it)->getHeight() : getTitleHeight());
+
+        _button_list.push_back(new PDecor::Button(_dpy, &_title_wo, *b_it, width, height));
         _button_list.back()->mapWindow();
         addChildWindow(_button_list.back()->getWindow());
     }
@@ -757,12 +793,12 @@ PDecor::loadDecor(void)
 
     // Make sure it gets rendered correctly.
     _dirty_resized = true;
-    _focused = !_focused;
-    setFocused(!_focused);
+    _focused = ! _focused;
+    setFocused(! _focused);
     _dirty_resized = false;
 
     // Update the dimension of the frame.
-    if (_child != NULL) {
+    if (_child) {
         resizeChild(_child->getWidth(), _child->getHeight());
     }
 
@@ -804,7 +840,7 @@ PDecor::findButton(Window win)
 PWinObj*
 PDecor::getChildFromPos(int x)
 {
-    if (!_child_list.size() || (_child_list.size() != _title_list.size()))
+    if (! _child_list.size() || (_child_list.size() != _title_list.size()))
         return NULL;
     if (_child_list.size() == 1)
         return _child_list.front();
@@ -849,19 +885,20 @@ PDecor::resizeChild(uint width, uint height)
 void
 PDecor::setBorder(StateAction sa)
 {
-    if (!ActionUtil::needToggle(sa, _border))
+    if (! ActionUtil::needToggle(sa, _border)) {
         return;
+    }
 
     // If we are going to remove the border, we need to check carefully
     // that we don't try to make the window disappear.
-    if (!_border) {
+    if (! _border) {
         _border = true;
-    } else if (!_shaded || _titlebar) {
+    } else if (! _shaded || _titlebar) {
         _border = false;
     }
 
     restackBorder();
-    if (!updateDecorName() && _child)
+    if (! updateDecorName() && _child)
         resizeChild(_child->getWidth(), _child->getHeight());
 }
 
@@ -869,40 +906,60 @@ PDecor::setBorder(StateAction sa)
 void
 PDecor::setTitlebar(StateAction sa)
 {
-    if (!ActionUtil::needToggle(sa, _titlebar))
+    if (! ActionUtil::needToggle(sa, _titlebar))
         return;
 
     // If we are going to remove the titlebar, we need to check carefully
     // that we don't try to make the window disappear.
-    if (!_titlebar) {
+    if (! _titlebar) {
         _title_wo.mapWindow();
         _titlebar = true;
-    } else if (!_shaded || _border) {
+    } else if (! _shaded || _border) {
         _title_wo.unmapWindow();
         _titlebar = false;
     }
 
     // If updateDecorName returns true, it allready loaded decor stuff for us.
-    if (!updateDecorName() && _child) {
+    if (! updateDecorName() && _child) {
         alignChild(_child);
         resizeChild(_child->getWidth(), _child->getHeight());
     }
 }
 
+//! @brief Calculate title height, 0 if titlebar is disabled.
+uint
+PDecor::getTitleHeight(void) const
+{
+    if (! _titlebar) {
+        return 0;
+    }
+
+    if (_data->isTitleHeightAdapt()) {
+        return getFont(getFocusedState(false))->getHeight()
+              + _data->getPad(PAD_UP) + _data->getPad(PAD_DOWN);
+    } else {
+        return _data->getTitleHeight();
+    }
+}
+
 //! @brief Adds a child to the decor, reparenting the window
 void
 PDecor::addChild(PWinObj *child)
 {
-    child->reparent(this, borderLeft(), borderRight() + getTitleHeight());
+    child->reparent(this, borderLeft(), borderTop() + getTitleHeight());
     _child_list.push_back(child);
 
     updatedChildOrder();
 
-    // if it's the first child, we'll want to initialize decor with
-    // shaping etc
+    // Sync focused state if it is the first child, the child will be
+    // activated later on. If there are children here already fit the
+    // child into the decor.
     if (_child_list.size() == 1) {
-        _focused = !_focused;
-        setFocused(!_focused);
+        _focused = ! _focused;
+        setFocused(! _focused);
+    } else {
+      alignChild(child);
+      child->resize(getChildWidth(), getChildHeight());
     }
 }
 
@@ -966,9 +1023,9 @@ PDecor::activateChild(PWinObj *child)
 
 //! @brief
 void
-PDecor::getDecorInfo(char *buf, uint size)
+PDecor::getDecorInfo(wchar_t *buf, uint size)
 {
-    snprintf(buf, size, "%dx%d+%d+%d", _gm.width, _gm.height, _gm.x, _gm.y);
+    swprintf(buf, size, L"%dx%d+%d+%d", _gm.width, _gm.height, _gm.x, _gm.y);
 }
 
 //! @brief
@@ -995,7 +1052,7 @@ void
 PDecor::activateChildRel(int off)
 {
     PWinObj *child = getChildRel(off);
-    if (child == NULL) {
+    if (! child) {
         child = _child_list.front();
     }
     activateChild(child);
@@ -1007,7 +1064,7 @@ void
 PDecor::moveChildRel(int off)
 {
     PWinObj *child = getChildRel(off);
-    if ((child == NULL) || (child == _child)) {
+    if (! child || (child == _child)) {
         return;
     }
 
@@ -1019,13 +1076,13 @@ PDecor::moveChildRel(int off)
     list<PWinObj*>::iterator it(find(_child_list.begin(), _child_list.end(), child));
     if (off > 0) {
         // when wrapping, we want to be able to insert at first place
-        if ((at_begin == true) || (*it != _child_list.front())) {
+        if (at_begin || (*it != _child_list.front())) {
             ++it;
         }
 
         _child_list.insert(it, _child);
     } else {
-        if ((at_end != true) && (*it == _child_list.back())) {
+        if (! at_end && (*it == _child_list.back())) {
             ++it;
         }
         _child_list.insert(it, _child);
@@ -1082,8 +1139,8 @@ PDecor::doMove(int x_root, int y_root)
     PScreen *scr = PScreen::instance(); // convenience
     StatusWindow *sw = StatusWindow::instance(); // convenience
 
-    if (scr->grabPointer(scr->getRoot(), ButtonMotionMask|ButtonReleaseMask,
-                         ScreenResources::instance()->getCursor(ScreenResources::CURSOR_MOVE)) == false) {
+    if (! scr->grabPointer(scr->getRoot(), ButtonMotionMask|ButtonReleaseMask,
+                         ScreenResources::instance()->getCursor(ScreenResources::CURSOR_MOVE))) {
         return;
     }
 
@@ -1091,7 +1148,7 @@ PDecor::doMove(int x_root, int y_root)
     int x = x_root - _gm.x;
     int y = y_root - _gm.y;
 
-    bool outline = (Config::instance()->getOpaqueMove() == false);
+    bool outline = ! Config::instance()->getOpaqueMove();
     EdgeType edge;
 
     // grab server, we don't want invert traces
@@ -1099,8 +1156,8 @@ PDecor::doMove(int x_root, int y_root)
         scr->grabServer();
     }
 
-    char buf[128];
-    getDecorInfo(buf, sizeof(buf)/sizeof(char));
+    wchar_t buf[128];
+    getDecorInfo(buf, 128);
 
     if (Config::instance()->isShowStatusWindow()) {
         sw->draw(buf, true);
@@ -1110,7 +1167,7 @@ PDecor::doMove(int x_root, int y_root)
 
     XEvent e;
     bool exit = false;
-    while (exit != true) {
+    while (! exit) {
         if (outline) {
             drawOutline(_gm);
         }
@@ -1125,7 +1182,7 @@ PDecor::doMove(int x_root, int y_root)
             _gm.y = e.xmotion.y_root - y;
             checkSnap();
 
-            if (outline == false) {
+            if (! outline) {
                 move(_gm.x, _gm.y);
             }
 
@@ -1134,7 +1191,7 @@ PDecor::doMove(int x_root, int y_root)
                 doMoveEdgeAction(&e.xmotion, edge);
             }
 
-            getDecorInfo(buf, sizeof(buf)/sizeof(char));
+            getDecorInfo(buf, 128);
             if (Config::instance()->isShowStatusWindow()) {
                 sw->draw(buf, true);
             }
@@ -1163,20 +1220,16 @@ PDecor::doMove(int x_root, int y_root)
 EdgeType
 PDecor::doMoveEdgeFind(int x, int y)
 {
-    if (Config::instance()->getScreenEdgeSize() == 0) {
-        return SCREEN_EDGE_NO;
-    }
-
     EdgeType edge = SCREEN_EDGE_NO;
-    if (x <= signed(Config::instance()->getScreenEdgeSize())) {
+    if (x <= signed(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_LEFT))) {
         edge = SCREEN_EDGE_LEFT;
     } else if (x >= signed(PScreen::instance()->getWidth() -
-                           Config::instance()->getScreenEdgeSize())) {
+                           Config::instance()->getScreenEdgeSize(SCREEN_EDGE_RIGHT))) {
         edge = SCREEN_EDGE_RIGHT;
-    } else if (y <= signed(Config::instance()->getScreenEdgeSize())) {
+    } else if (y <= signed(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_TOP))) {
         edge = SCREEN_EDGE_TOP;
     } else if (y >= signed(PScreen::instance()->getHeight() -
-                           Config::instance()->getScreenEdgeSize())) {
+                           Config::instance()->getScreenEdgeSize(SCREEN_EDGE_BOTTOM))) {
         edge = SCREEN_EDGE_BOTTOM;
     }
 
@@ -1194,7 +1247,7 @@ PDecor::doMoveEdgeAction(XMotionEvent *ev, EdgeType edge)
                                         MOUSE_EVENT_ENTER_MOVING,
                                         Config::instance()->getEdgeListFromPosition(edge));
 
-    if (ae != NULL) {
+    if (ae) {
         ActionPerformed ap(this, *ae);
         ap.type = ev->type;
         ap.event.motion = ev;
@@ -1210,23 +1263,23 @@ PDecor::doKeyboardMoveResize(void)
     PScreen *scr = PScreen::instance(); // convenience
     StatusWindow *sw = StatusWindow::instance(); // convenience
 
-    if (scr->grabPointer(scr->getRoot(), NoEventMask,
-                         ScreenResources::instance()->getCursor(ScreenResources::CURSOR_MOVE)) == false) {
+    if (! scr->grabPointer(scr->getRoot(), NoEventMask,
+                         ScreenResources::instance()->getCursor(ScreenResources::CURSOR_MOVE))) {
         return;
     }
-    if (scr->grabKeyboard(scr->getRoot()) == false) {
+    if (! scr->grabKeyboard(scr->getRoot())) {
         scr->ungrabPointer();
         return;
     }
 
     Geometry old_gm = _gm; // backup geometry if we cancel
-    bool outline = ((Config::instance()->getOpaqueMove() == false) ||
-                    (Config::instance()->getOpaqueResize() == false));
+    bool outline = (! Config::instance()->getOpaqueMove() ||
+                    ! Config::instance()->getOpaqueResize());
     ActionEvent *ae;
     list<Action>::iterator it;
 
-    char buf[128];
-    getDecorInfo(buf, sizeof(buf)/sizeof(char));
+    wchar_t buf[128];
+    getDecorInfo(buf, 128);
 
     if (Config::instance()->isShowStatusWindow()) {
         sw->draw(buf, true);
@@ -1240,7 +1293,7 @@ PDecor::doKeyboardMoveResize(void)
 
     XEvent e;
     bool exit = false;
-    while (exit != true) {
+    while (! exit) {
         if (outline) {
             drawOutline(_gm);
         }
@@ -1249,43 +1302,43 @@ PDecor::doKeyboardMoveResize(void)
             drawOutline(_gm); // clear
         }
 
-        if ((ae = KeyGrabber::instance()->findMoveResizeAction(&e.xkey)) != NULL) {
+        if ((ae = KeyGrabber::instance()->findMoveResizeAction(&e.xkey)) != 0) {
             for (it = ae->action_list.begin(); it != ae->action_list.end(); ++it) {
                 switch (it->getAction()) {
                 case MOVE_HORIZONTAL:
                     _gm.x += it->getParamI(0);
-                    if (outline == false) {
+                    if (! outline) {
                         move(_gm.x, _gm.y);
                     }
                     break;
                 case MOVE_VERTICAL:
                     _gm.y += it->getParamI(0);
-                    if (outline == false) {
+                    if (! outline) {
                         move(_gm.x, _gm.y);
                     }
                     break;
                 case RESIZE_HORIZONTAL:
                     _gm.width += resizeHorzStep(it->getParamI(0));
-                    if (outline == false) {
+                    if (! outline) {
                         resize(_gm.width, _gm.height);
                     }
                     break;
                 case RESIZE_VERTICAL:
                     _gm.height += resizeVertStep(it->getParamI(0));
-                    if (outline == false) {
+                    if (! outline) {
                         resize(_gm.width, _gm.height);
                     }
                     break;
                 case MOVE_SNAP:
                     checkSnap();
-                    if (outline == false) {
+                    if (! outline) {
                         move(_gm.x, _gm.y);
                     }
                     break;
                 case MOVE_CANCEL:
                     _gm = old_gm; // restore position
 
-                    if (outline == false) {
+                    if (! outline) {
                         move(_gm.x, _gm.y);
                         resize(_gm.width, _gm.height);
                     }
@@ -1306,7 +1359,7 @@ PDecor::doKeyboardMoveResize(void)
                     break;
                 }
 
-                getDecorInfo(buf, sizeof(buf)/sizeof(char));
+                getDecorInfo(buf, 128);
                 if (Config::instance()->isShowStatusWindow()) {
                     sw->draw(buf, true);
                 }
@@ -1330,7 +1383,7 @@ PDecor::doKeyboardMoveResize(void)
 void
 PDecor::setShaded(StateAction sa)
 {
-    if (!ActionUtil::needToggle(sa, _shaded))
+    if (! ActionUtil::needToggle(sa, _shaded))
         return;
 
     // If we are going to shade the window, we need to check carefully
@@ -1340,16 +1393,17 @@ PDecor::setShaded(StateAction sa)
         _shaded = false;
         // If we have a titlebar OR border (border will only be visible in
         // full-width mode only)
-    } else if (_titlebar || (_border && !_data->getTitleWidthMin())) {
+    } else if (_titlebar || (_border && ! _data->getTitleWidthMin())) {
         _real_height = _gm.height;
         _gm.height = getTitleHeight();
         // shading in non full width title mode will make border go away
-        if (!_data->getTitleWidthMin())
+        if (! _data->getTitleWidthMin())
             _gm.height += borderTop() + borderBottom();
         _shaded = true;
     }
 
     PWinObj::resize(_gm.width, _gm.height);
+    setBorderShape();
     placeBorder();
     restackBorder();
 }
@@ -1365,7 +1419,7 @@ PDecor::setSkip(uint skip)
 void
 PDecor::renderTitle(void)
 {
-    if (!getTitleHeight()) {
+    if (! getTitleHeight()) {
         return;
     }
 
@@ -1417,16 +1471,18 @@ PDecor::renderTitle(void)
         font = getFont(getFocusedState(sel));
         font->setColor(_data->getFontColor(getFocusedState(sel)));
 
-        // this is a icky one, draws the text in the tab depending on justify
-        // and wheter or not the title has a TitleRule applied
-        if ((*it) != NULL) {
-            font->draw(_title_bg,
-                       x + _data->getPad(PAD_LEFT), // X position
-                       _data->getPad(PAD_UP), // Y position
-                       (*it)->getVisible().c_str(), 0, // Text and max chars
-                       (*it)->getWidth() - pad_horiz, // Available width
-                       ((*it)->isCustom() || (*it)->isUserSet()) // Type of trimming
-                       ? PFont::FONT_TRIM_END : PFont::FONT_TRIM_MIDDLE);
+        if ((*it)) {
+          PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
+          if ((*it)->isCustom() || (*it)->isUserSet()) {
+            trim = PFont::FONT_TRIM_END;
+          }
+
+          font->draw(_title_bg,
+                     x + _data->getPad(PAD_LEFT), // X position
+                     _data->getPad(PAD_UP), // Y position
+                     (*it)->getVisible(), 0, // Text and max chars
+                     (*it)->getWidth() - pad_horiz, // Available width
+                     trim); // Type of trim
         }
 
         // move to next tab (or separator if any)
@@ -1456,7 +1512,7 @@ PDecor::renderButtons(void)
 void
 PDecor::renderBorder(void)
 {
-    if (!_border) {
+    if (! _border) {
         return;
     }
 
@@ -1503,37 +1559,37 @@ PDecor::renderBorder(void)
     }
 }
 
-//! @brief
+//! @brief Sets shape on the border windows.
 void
 PDecor::setBorderShape(void)
 {
 #ifdef HAVE_SHAPE
-    uint pos[] = { BORDER_TOP_LEFT, BORDER_TOP_RIGHT,
-                   BORDER_BOTTOM_LEFT, BORDER_BOTTOM_RIGHT };
-    bool f;
     Pixmap pix;
+    bool do_free;
+    unsigned int width, height;
     FocusedState state = getFocusedState(false);
 
-    XRectangle rect;
-    rect.x = 0;
-    rect.y = 0;
+    XRectangle rect = {0, 0, 0, 0};
 
-    for (uint i = pos[0]; i < (sizeof(pos)/sizeof(pos[0])); ++i) {
-        pix =
-            _data->getBorderTexture(state, BorderPosition(pos[i]))->getMask(0, 0, f);
+    map<BorderPosition, Pixmap>::iterator it(_border_pos_map.begin());
+    for (; it != _border_pos_map.end(); ++it) {
+        // Get the size of the border at position
+        getBorderSize(it->first, width, height);
+
+        // Get shape pixmap
+        pix =  _data->getBorderTexture(state, it->first)->getMask(width, height,
+                                                                  do_free);
         if (pix != None) {
             _need_shape = true;
-            XShapeCombineMask(_dpy, _border_win[pos[i]],
-                              ShapeBounding, 0, 0, pix, ShapeSet);
-            if (f) {
+            XShapeCombineMask(_dpy, _border_win[it->first],
+                            ShapeBounding, 0, 0, pix, ShapeSet);
+            if (do_free) {
                 ScreenResources::instance()->getPixmapHandler()->returnPixmap(pix);
             }
         } else {
-            rect.width =
-                _data->getBorderTexture(state, BorderPosition(pos[i]))->getWidth();
-            rect.height =
-                _data->getBorderTexture(state, BorderPosition(pos[i]))->getHeight();
-            XShapeCombineRectangles(_dpy, _border_win[pos[i]], ShapeBounding,
+            rect.width = width;
+            rect.height = height;
+            XShapeCombineRectangles(_dpy, _border_win[it->first], ShapeBounding,
                                     0, 0, &rect, 1, ShapeSet, YXBanded);
         }
     }
@@ -1544,8 +1600,7 @@ PDecor::setBorderShape(void)
 uint
 PDecor::getNearestHead(void)
 {
-    return PScreen::instance()->getNearestHead(_gm.x + (_gm.width / 2),
-            _gm.y + (_gm.height / 2));
+    return PScreen::instance()->getNearestHead(_gm.x + (_gm.width / 2), _gm.y + (_gm.height / 2));
 }
 
 //! @brief
@@ -1562,7 +1617,7 @@ PDecor::checkSnap(void)
     }
 }
 
-inline bool // FIXME: hmm
+inline bool
 isBetween(int x1, int x2, int t1, int t2)
 {
     if (x1 > t1) {
@@ -1593,8 +1648,7 @@ PDecor::checkWOSnap(void)
 
     vector<PWinObj*>::reverse_iterator it = _wo_list.rbegin();
     for (; it != _wo_list.rend(); ++it) {
-        if (((*it) == this) || ! (*it)->isMapped()
-                || ((*it)->getType() != PWinObj::WO_FRAME)) {
+        if (((*it) == this) || ! (*it)->isMapped() || ((*it)->getType() != PWinObj::WO_FRAME)) {
             continue;
         }
 
@@ -1646,7 +1700,7 @@ PDecor::checkEdgeSnap(void)
     int resist = Config::instance()->getEdgeResist();
 
     Geometry head;
-    PScreen::instance()->getHeadInfo(PScreen::instance()->getNearestHead(_gm.x, _gm.y), head);
+    PScreen::instance()->getHeadInfoWithEdge(PScreen::instance()->getNearestHead(_gm.x, _gm.y), head);
 
     if ((_gm.x >= (head.x - resist)) && (_gm.x <= (head.x + attract))) {
         _gm.x = head.x;
@@ -1668,7 +1722,7 @@ PDecor::checkEdgeSnap(void)
 bool
 PDecor::setShape(void)
 {
-    if (!_child) {
+    if (! _child) {
         return false;
     }
 
@@ -1693,7 +1747,7 @@ PDecor::setShape(void)
 void
 PDecor::alignChild(PWinObj *child)
 {
-    if (child == NULL) {
+    if (! child) {
         return;
     }
 
@@ -1819,7 +1873,7 @@ PDecor::applyBorderShape(void)
         shape = XCreateSimpleWindow(_dpy, PScreen::instance()->getRoot(),
                                     0, 0, _gm.width, _gm.height, 0, 0, 0);
 
-        if (_child != NULL) {
+        if (_child) {
             XShapeCombineShape(_dpy, shape, ShapeBounding,
                                borderLeft(), borderTop() + getTitleHeight(),
                                _child->getWindow(), ShapeBounding, ShapeSet);
@@ -1827,8 +1881,8 @@ PDecor::applyBorderShape(void)
 
         // Apply border shape. Need to be carefull wheter or not to include it.
         if (_border
-                && !(_shaded && bt_off) // Shaded in non-full-width mode removes border.
-                && !(_need_client_shape)) { // Shaped clients should appear bordeless.
+                && ! (_shaded && bt_off) // Shaded in non-full-width mode removes border.
+                && ! (_need_client_shape)) { // Shaped clients should appear bordeless.
             // top
             if (borderTop() > 0) {
                 XShapeCombineShape(_dpy, shape, ShapeBounding,
@@ -1847,25 +1901,19 @@ PDecor::applyBorderShape(void)
                                    ShapeBounding, ShapeUnion);
             }
 
-            // y position for left and right border
-            int side_y;
-            if (!bt_off && !borderTop())
-                side_y = getTitleHeight();
-            else
-                side_y = bt_off + borderTopLeftHeight();
-
-            // left
+            bool use_bt_off = bt_off || borderTop();
+            // Left border
             if (borderLeft() > 0) {
                 XShapeCombineShape(_dpy, shape, ShapeBounding,
-                                   0, side_y,
+                                   0, use_bt_off ? bt_off + borderTopLeftHeight() : getTitleHeight(),
                                    _border_win[BORDER_LEFT],
                                    ShapeBounding, ShapeUnion);
             }
 
-            // right
+            // Right border
             if (borderRight() > 0) {
                 XShapeCombineShape(_dpy, shape, ShapeBounding,
-                                   _gm.width - borderRight(), side_y,
+                                   _gm.width - borderRight(), use_bt_off ? bt_off + borderTopRightHeight() : getTitleHeight(),
                                    _border_win[BORDER_RIGHT],
                                    ShapeBounding, ShapeUnion);
             }
@@ -1924,7 +1972,7 @@ PDecor::restackBorder(void)
 
 
     // Only put the Child over if not shaded.
-    if (_child && !_shaded) {
+    if (_child && ! _shaded) {
         windows[extra++] = _child->getWindow();
     }
     // Add title if any
@@ -2038,7 +2086,7 @@ PDecor::calcTitleWidth(void)
         if (_data->isTitleWidthSymetric()) {
             list<PDecor::TitleItem*>::iterator it(_title_list.begin());
             for (; it != _title_list.end(); ++it) {
-                width = font->getWidth((*it)->getVisible().c_str());
+                width = font->getWidth((*it)->getVisible());
                 if (width > width_max)
                     width_max = width;
             }
@@ -2051,8 +2099,8 @@ PDecor::calcTitleWidth(void)
         } else {
             list<PDecor::TitleItem*>::iterator it(_title_list.begin());
             for (; it != _title_list.end(); ++it) {
-                width += font->getWidth((*it)->getVisible().c_str())
-                         + _data->getPad(PAD_LEFT)	+ _data->getPad(PAD_RIGHT);
+                width += font->getWidth((*it)->getVisible())
+                         + _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
             }
         }
 
@@ -2077,7 +2125,7 @@ PDecor::calcTitleWidth(void)
 void
 PDecor::calcTabsWidth(void)
 {
-    if (!_title_list.size())
+    if (! _title_list.size())
         return;
 
     if (_data->isTitleWidthSymetric()) {
