@@ -1,35 +1,38 @@
 //
 // PFont.cc for pekwm
-// Copyright (C) 2003-2005 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2003-2008 Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
-#include "PFont.hh"
-
-#include "PScreen.hh"
-#include "Util.hh"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
 
 #include <iostream>
 #include <cstring>
 
+#include "PFont.hh"
+#include "PScreen.hh"
+#include "Util.hh"
+
 using std::cerr;
 using std::endl;
 using std::string;
 using std::vector;
+using std::wstring;
 
-string PFont::_trim_string = string();
+wstring PFont::_trim_string = wstring();
 const char *FALLBACK_FONT = "fixed";
 
 // PFont
 
 //! @brief PFont constructor
 PFont::PFont(PScreen *scr) :
-        _scr(scr),
-        _height(0), _ascent(0), _descent(0),
-        _offset_x(0), _offset_y(0), _justify(FONT_JUSTIFY_LEFT)
+  _scr(scr),
+  _height(0), _ascent(0), _descent(0),
+  _offset_x(0), _offset_y(0), _justify(FONT_JUSTIFY_LEFT)
 {
     _type = FONT_TYPE_NO;
 }
@@ -37,6 +40,7 @@ PFont::PFont(PScreen *scr) :
 //! @brief PFont destructor
 PFont::~PFont(void)
 {
+    unload();
 }
 
 //! @brief Draws the text on the drawable.
@@ -48,139 +52,122 @@ PFont::~PFont(void)
 //! @param max_width max nr of pixels, defaults to 0 == infinity
 //! @param trim_type how to trim title if not enough space, defaults to FONT_TRIM_END
 void
-PFont::draw(Drawable dest, int x, int y, const char *text, uint max_chars, uint max_width, PFont::TrimType trim_type)
+PFont::draw(Drawable dest, int x, int y, const std::wstring &text,
+            uint max_chars, uint max_width, PFont::TrimType trim_type)
 {
-    if (text == NULL) {
-        return;
+    if (! text.size()) {
+        cerr << " *** WARNING: trying to draw empty string." << endl;
     }
 
     uint offset = x, chars = max_chars;
-    bool free_text = false;
-    char *real_text = (char*) text;
+    wstring real_text(text);
 
-    // if we have max width set, make sure our text isn't longer
     if (max_width > 0) {
-        free_text = trim(&real_text, trim_type, max_width, chars);
+        // If max width set, make sure text fits in max_width pixels.
+        trim(real_text, trim_type, max_width);
+
         offset += justify(real_text, max_width, 0,
-                          (chars == 0) ? strlen(real_text) : chars);
-        // chars will be set in trim if calling that
+                      (chars == 0) ? real_text.size() : chars);
     } else if (chars == 0) {
-        chars = strlen(real_text);
+        // Just set to complete string.
+        chars = real_text.size();
     }
 
-    // shadowed font
-    if ((_offset_x != 0) || (_offset_y != 0)) {
+    // Draw shadowed font if x or y offset is specified
+    if (_offset_x || _offset_y) {
         drawText(dest, offset + _offset_x, y + _ascent + _offset_y,
-                 real_text, chars, false); // false as in bg
+                real_text, chars, false); // false as in bg
     }
 
-    // fg text
+    // Draw main font
     drawText(dest, offset, y + _ascent, real_text, chars, true); // true as in fg
-
-    if (free_text) {
-        free(real_text);
-    }
 }
 
 //! @brief Trims the text making it max max_width wide
-//! @return true if *text needs to be free, else false
-bool
-PFont::trim(char **text, PFont::TrimType trim_type, uint max_width, uint &chars)
+//! @param text
+//! @param trim_type
+//! @param max_width
+//! @param chars
+void
+PFont::trim(std::wstring &text, PFont::TrimType trim_type, uint max_width)
 {
-    bool free_text = false;
-
-    if (getWidth(*text) > max_width) {
+    if (getWidth(text) > max_width) {
         if ((_trim_string.size() > 0) && (trim_type == FONT_TRIM_MIDDLE)) {
-            *text = trimMiddle(*text, max_width);
-            chars = trimEnd(*text, max_width);
-
-            free_text = true;
-        } else {
-            chars = trimEnd(*text, max_width);
+            // Trim middle before end if trim string is set.
+            trimMiddle(text, max_width);
         }
-    } else {
-        chars = strlen(*text);
-    }
 
-    return free_text;
+        trimEnd(text, max_width);
+    }
 }
 
 //! @brief Figures how many charachters to have before exceding max_width
-uint
-PFont::trimEnd(const char *text, uint max_width)
+void
+PFont::trimEnd(std::wstring &text, uint max_width)
 {
-    for (uint i = strlen(text); i > 0; --i) {
+    for (uint i = text.size(); i > 0; --i) {
         if (getWidth(text, i) <= max_width) {
-            return i;
+            text = text.substr(0, i);
+            break;
         }
     }
-
-    return 0;
 }
 
 //! @brief Replace middle of string with _trim_string making it max_width wide
-char*
-PFont::trimMiddle(const char *text, uint max_width)
+void
+PFont::trimMiddle(std::wstring &text, uint max_width)
 {
-    if (_trim_string.size() == 0) {
-        return strdup(text);
-    }
-
-    string dest;
-    uint chars = strlen(text), pos = 0;
-
-    // max width
+    // Get max and separator width
     uint max_side = (max_width / 2);
     uint sep_width = getWidth(_trim_string.c_str(),
-                              _trim_string.size() / 2 + _trim_string.size() % 2);
+                            _trim_string.size() / 2 + _trim_string.size() % 2);
 
-    // if we can't hold the trim string, just do an trim end
+    uint pos = 0;
+    wstring dest;
+
+    // If the trim string is too large, do nothing and let trimEnd handle this.
     if (max_side <= sep_width) {
-        return strdup(text);
+        return;
     }
-    // add space for the trim string
+    // Add space for the trim string
     max_side -= sep_width;
 
-    // get numbers of chars before ...
-    for (uint i = chars; i > 0; --i) {
+    // Get numbers of chars before trim string (..)
+    for (uint i = text.size(); i > 0; --i) {
         if (getWidth(text, i) < max_side) {
             pos = i;
-            dest.insert(0, text, i);
+            dest.insert(0, text.substr(0, i));
             break;
         }
     }
+    
     // get numbers of chars after ...
-    for (uint i = pos; i < chars; ++i) {
-        if (getWidth(text + i, chars - i) < max_side) {
-            dest.insert(dest.size(), text + i, chars - i);
-            break;
+    if (pos < text.size()) {
+        for (uint i = pos; i < text.size(); ++i) {
+            wstring second_part(text.substr(i, text.size() - i));
+            if (getWidth(second_part, 0) < max_side) {
+                dest.insert(dest.size(), second_part);
+                break;
+            }
+        }
+
+        // Got a char after and before, if not do nothing and trimEnd will handle
+        // trimming after this call.
+        if (dest.size() > 1) {
+            if ((getWidth(dest) + getWidth(_trim_string)) < max_width) {
+                dest.insert(pos, _trim_string);
+            }
+
+            // Update original string
+            text = dest;
         }
     }
-
-    // got a char after and before
-    if (dest.size() > 1) {
-        if ((getWidth(dest.c_str()) + getWidth(_trim_string.c_str())) < max_width) {
-            dest.insert(pos, _trim_string);
-        }
-        // not enough room to place a characther in max_side, do trim end instead
-    } else {
-        return strdup(text);
-    }
-
-#ifdef DEBUG
-    if (getWidth(dest.c_str()) > max_width) {
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "width(" << getWidth(dest.c_str())
-             << ") is greater than max_width(" << max_width << ")" << endl;
-    }
-#endif // DEBUG
-
-    return strdup(dest.c_str());
 }
 
 //! @brief Justifies the string based on _justify property of the Font
 uint
-PFont::justify(const char *text, uint max_width, uint padding, uint chars)
+PFont::justify(const std::wstring &text, uint max_width,
+               uint padding, uint chars)
 {
     uint x;
     uint width = getWidth(text, chars);
@@ -223,9 +210,9 @@ PFontX11::load(const std::string &font_name)
     unload();
 
     _font = XLoadQueryFont(_scr->getDpy(), font_name.c_str());
-    if (_font == NULL) {
+    if (! _font) {
         _font = XLoadQueryFont(_scr->getDpy(), FALLBACK_FONT);
-        if (_font == NULL) {
+        if (! _font) {
             return false;
         }
     }
@@ -250,7 +237,6 @@ PFontX11::load(const std::string &font_name)
 void
 PFontX11::unload(void)
 {
-
     if (_font) {
         XFreeFont(_scr->getDpy(), _font);
         _font = NULL;
@@ -267,24 +253,51 @@ PFontX11::unload(void)
 
 //! @brief Gets the width the text would take using this font
 uint
-PFontX11::getWidth(const char *text, uint max_chars)
+PFontX11::getWidth(const wstring &text, uint max_chars)
 {
-    if (text == NULL)
+    if (! text.size()) {
         return 0;
+    }
 
-    // make sure the max_chars has a decent value, if it's less than 1 wich it
+    // Make sure the max_chars has a decent value, if it's less than 1 wich it
     // defaults to set it to the numbers of chars in the string text
-    if ((max_chars == 0) || (max_chars > strlen(text)))
-        max_chars = strlen(text);
-
+    if (! max_chars || (max_chars > text.size())) {
+        max_chars = text.size();
+    }
+  
     uint width = 0;
-    if (_font != NULL) {
-        width = XTextWidth(_font, text, max_chars);
+    if (_font) {
+        // No UTF8 support, convert to locale encoding.
+        string mb_text(Util::to_mb_str(text.substr(0, max_chars)));
+        width = XTextWidth(_font, mb_text.c_str(), mb_text.size());
     }
 
     return (width + _offset_x);
 }
 
+//! @brief Draws the text on dest
+//! @param dest Drawable to draw on
+//! @param chars Max numbers of characthers to print
+//! @param fg Bool, to use foreground or background colors
+void
+PFontX11::drawText(Drawable dest, int x, int y,
+                   const std::wstring &text, uint chars, bool fg)
+{
+    GC gc = fg ? _gc_fg : _gc_bg;
+
+    if (_font && (gc != None)) {
+        string mb_text;
+        if (chars != 0) {
+            mb_text = Util::to_mb_str(text.substr(0, chars));
+        } else {
+            mb_text = Util::to_mb_str(text);
+        }
+
+        XDrawString(_scr->getDpy(), dest, gc, x, y,
+                mb_text.c_str(), mb_text.size());
+    }
+}
+
 //! @brief Sets the color that should be used when drawing
 void
 PFontX11::setColor(PFont::Color *color)
@@ -298,26 +311,14 @@ PFontX11::setColor(PFont::Color *color)
     }
 }
 
-//! @brief Draws the text on dest
-//! @param dest Drawable to draw on
-//! @param chars Max numbers of characthers to print
-//! @param fg Bool, to use foreground or background colors
-void
-PFontX11::drawText(Drawable dest, int x, int y, const char *text, uint chars, bool fg)
-{
-    GC gc = fg ? _gc_fg : _gc_bg;
-
-    if (_font && (gc != None)) {
-        XDrawString(_scr->getDpy(), dest, gc, x, y, text, chars);
-    }
-}
-
 // PFontXmb
 
+const char* PFontXmb::DEFAULT_FONTSET = "fixed*";
+
 //! @brief PFontXmb constructor
-PFontXmb::PFontXmb(PScreen *scr) : PFont(scr),
-        _fontset(NULL),
-        _gc_fg(None), _gc_bg(None)
+PFontXmb::PFontXmb(PScreen *scr)
+  : PFont(scr),
+    _fontset(NULL), _gc_fg(None), _gc_bg(None)
 {
     _type = FONT_TYPE_XMB;
 }
@@ -334,38 +335,12 @@ PFontXmb::load(const std::string &font_name)
 {
     unload();
 
-    char *def_string;
-    char **missing_list, **font_names;
-    int missing_count, fnum;
-    XFontStruct **fonts;
-
     string basename(font_name);
-    basename.append(",*");
-
-    _fontset =
-        XCreateFontSet(_scr->getDpy(), basename.c_str(),
-                       &missing_list, &missing_count, &def_string);
-
-    if (_fontset == NULL) {
-        return false;
+    if (font_name.rfind(",*") != (font_name.size() - 2)) {
+        basename.append(",*");
     }
 
-    for (int i = 0; i < missing_count; ++i)
-        cerr <<  "PFontXmb::load(): Missing charset" <<  missing_list[i] << endl;
-
-    _ascent = _descent = 0;
-
-    fnum = XFontsOfFontSet (_fontset, &fonts, &font_names);
-    for (int i = 0; i < fnum; ++i) {
-        if (signed(_ascent) < fonts[i]->ascent)
-            _ascent = fonts[i]->ascent;
-        if (signed(_descent) < fonts[i]->descent)
-            _descent = fonts[i]->descent;
-    }
-
-    _height = _ascent + _descent;
-
-    // create GC's
+    // Create GC
     XGCValues gv;
 
     uint mask = GCFunction;
@@ -373,6 +348,41 @@ PFontXmb::load(const std::string &font_name)
     _gc_fg = XCreateGC(_scr->getDpy(), _scr->getRoot(), mask, &gv);
     _gc_bg = XCreateGC(_scr->getDpy(), _scr->getRoot(), mask, &gv);
 
+    // Try to load font
+    char *def_string;
+    char **missing_list, **font_names;
+    int missing_count;
+
+    _fontset = XCreateFontSet(_scr->getDpy(), basename.c_str(), &missing_list, &missing_count, &def_string);
+    if (! _fontset) {
+        cerr << "PFontXmb::load(): Failed to create fontset for " << font_name << endl;
+        _fontset = XCreateFontSet(_scr->getDpy(), DEFAULT_FONTSET, &missing_list, &missing_count, &def_string);
+    }
+
+    if (_fontset) {
+        for (int i = 0; i < missing_count; ++i) {
+            cerr <<  "PFontXmb::load(): Missing charset" <<  missing_list[i] << endl;
+        }
+
+        _ascent = _descent = 0;
+
+        XFontStruct **fonts;
+        int fnum = XFontsOfFontSet (_fontset, &fonts, &font_names);
+        for (int i = 0; i < fnum; ++i) {
+            if (signed(_ascent) < fonts[i]->ascent) {
+                _ascent = fonts[i]->ascent;
+            }
+            if (signed(_descent) < fonts[i]->descent) {
+                _descent = fonts[i]->descent;
+            }
+        }
+
+        _height = _ascent + _descent;
+
+    } else {
+        return false;
+    }
+
     return true;
 }
 
@@ -380,7 +390,7 @@ PFontXmb::load(const std::string &font_name)
 void
 PFontXmb::unload(void)
 {
-    if (_fontset != NULL) {
+    if (_fontset) {
         XFreeFontSet(_scr->getDpy(), _fontset);
         _fontset = NULL;
     }
@@ -388,20 +398,22 @@ PFontXmb::unload(void)
 
 //! @brief Gets the width the text would take using this font
 uint
-PFontXmb::getWidth(const char *text, uint max_chars)
+PFontXmb::getWidth(const std::wstring &text, uint max_chars)
 {
-    if (text == NULL)
+    if (! text.size()) {
         return 0;
+    }
 
-    // make sure the max_chars has a decent value, if it's less than 1 wich it
+    // Make sure the max_chars has a decent value, if it's less than 1 wich it
     // defaults to set it to the numbers of chars in the string text
-    if ((max_chars == 0) || (max_chars > strlen(text)))
-        max_chars = strlen(text);
+    if (! max_chars || (max_chars > text.size())) {
+        max_chars = text.size();
+    }
 
     uint width = 0;
-    if (_fontset != NULL) {
+    if (_fontset) {
         XRectangle ink, logical;
-        XmbTextExtents(_fontset, text, max_chars, &ink, &logical);
+        XwcTextExtents(_fontset, text.c_str(), max_chars, &ink, &logical);
         width = logical.width;
     }
 
@@ -413,12 +425,14 @@ PFontXmb::getWidth(const char *text, uint max_chars)
 //! @param chars Max numbers of characthers to print
 //! @param fg Bool, to use foreground or background colors
 void
-PFontXmb::drawText(Drawable dest, int x, int y, const char *text, uint chars, bool fg)
+PFontXmb::drawText(Drawable dest, int x, int y,
+                   const std::wstring &text, uint chars, bool fg)
 {
     GC gc = fg ? _gc_fg : _gc_bg;
 
-    if ((_fontset != NULL) && (gc != None)) {
-        XmbDrawString(_scr->getDpy(), dest, _fontset, gc, x, y, text, chars);
+    if (_fontset && (gc != None)) {
+        XwcDrawString(_scr->getDpy(), dest, _fontset, gc,
+                  x, y, text.c_str(), chars ? chars : text.size());
     }
 }
 
@@ -452,15 +466,20 @@ PFontXft::PFontXft(PScreen *scr) : PFont(scr),
 //! @brief PFontXft destructor
 PFontXft::~PFontXft(void)
 {
+    unload();
+
     XftDrawDestroy(_draw);
 
-    if (_cl_fg != NULL) {
+    if (_cl_fg) {
         XftColorFree(_scr->getDpy(), _scr->getVisual()->getXVisual(),
                      _scr->getColormap(), _cl_fg);
+        delete _cl_fg;
     }
-    if (_cl_bg != NULL) {
+    
+    if (_cl_bg) {
         XftColorFree(_scr->getDpy(), _scr->getVisual()->getXVisual(),
                      _scr->getColormap(), _cl_bg);
+        delete _cl_bg;
     }
 }
 
@@ -472,12 +491,12 @@ PFontXft::load(const std::string &font_name)
 
     _font = XftFontOpenName(_scr->getDpy(), _scr->getScreenNum(),
                             font_name.c_str());
-    if (_font == NULL) {
+    if (! _font) {
         _font = XftFontOpenXlfd(_scr->getDpy(), _scr->getScreenNum(),
                                 font_name.c_str());
     }
 
-    if (_font != NULL) {
+    if (_font) {
         _ascent = _font->ascent;
         _descent = _font->descent;
         _height = _font->height;
@@ -491,7 +510,7 @@ PFontXft::load(const std::string &font_name)
 void
 PFontXft::unload(void)
 {
-    if (_font != NULL) {
+    if (_font) {
         XftFontClose(_scr->getDpy(), _font);
         _font = NULL;
     }
@@ -499,21 +518,28 @@ PFontXft::unload(void)
 
 //! @brief Gets the width the text would take using this font
 uint
-PFontXft::getWidth(const char *text, uint max_chars)
+PFontXft::getWidth(const std::wstring &text, uint max_chars)
 {
-    uint width = 0;
+    if (! text.size()) {
+        return 0;
+    }
 
-    // make sure the max_chars has a decent value, if it's less than 1 wich it
+    // Make sure the max_chars has a decent value, if it's less than 1 wich it
     // defaults to set it to the numbers of chars in the string text
-    if ((max_chars > strlen(text)) || (max_chars < 1))
-        max_chars = strlen(text);
+    if (! max_chars || (max_chars > text.size())) {
+        max_chars = text.size();
+    }
+
+    uint width = 0;
+    if (_font) {
+        string utf8_text(Util::to_utf8_str(text.substr(0, max_chars)));
 
-    if (_font != NULL) {
         XGlyphInfo extents;
-        XftTextExtents8(_scr->getDpy(), _font,
-                        (XftChar8 *) text, max_chars, &extents);
+        XftTextExtentsUtf8(_scr->getDpy(), _font,
+                         reinterpret_cast<const XftChar8*>(utf8_text.c_str()),
+                         utf8_text.size(), &extents);
 
-        width = extents.width;
+        width = extents.xOff;
     }
 
     return (width + _offset_x);
@@ -524,13 +550,23 @@ PFontXft::getWidth(const char *text, uint max_chars)
 //! @param chars Max numbers of characthers to print
 //! @param fg Bool, to use foreground or background colors
 void
-PFontXft::drawText(Drawable dest, int x, int y, const char *text, uint chars, bool fg)
+PFontXft::drawText(Drawable dest, int x, int y,
+                   const std::wstring &text, uint chars, bool fg)
 {
     XftColor *cl = fg ? _cl_fg : _cl_bg;
 
-    if ((_font != NULL) && (cl != NULL)) {
+    if (_font && cl) {
+        string utf8_text;
+        if (chars != 0) {
+            utf8_text = Util::to_utf8_str(text.substr(0, chars));
+        } else {
+            utf8_text = Util::to_utf8_str(text);
+        }
+
         XftDrawChange(_draw, dest);
-        XftDrawStringUtf8(_draw, cl, _font, x, y, (XftChar8 *) text, chars);
+        XftDrawStringUtf8(_draw, cl, _font, x, y,
+                          reinterpret_cast<const XftChar8*>(utf8_text.c_str()),
+                          utf8_text.size());
     }
 }
 
@@ -538,14 +574,16 @@ PFontXft::drawText(Drawable dest, int x, int y, const char *text, uint chars, bo
 void
 PFontXft::setColor(PFont::Color *color)
 {
-    if (_cl_fg != NULL) {
+    if (_cl_fg) {
         XftColorFree(_scr->getDpy(), _scr->getVisual()->getXVisual(),
                      _scr->getColormap(), _cl_fg);
+        delete _cl_fg;
         _cl_fg = NULL;
     }
-    if (_cl_bg != NULL) {
+    if (_cl_bg) {
         XftColorFree(_scr->getDpy(), _scr->getVisual()->getXVisual(),
                      _scr->getColormap(), _cl_bg);
+        delete _cl_bg;
         _cl_bg = NULL;
     }
 
