@@ -31,7 +31,7 @@ StatusWindow::StatusWindow(Display *dpy, Theme *theme) :
         PDecor(dpy, theme, "STATUSWINDOW"),
         _bg(None)
 {
-    if (_instance != NULL) {
+    if (_instance) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "StatusWindow(" << this << ")::StatusWindow()" << endl
@@ -86,7 +86,7 @@ StatusWindow::~StatusWindow(void)
 //! @param text Text to draw in StatusWindow
 //! @param do_center Center the StatusWindow on screen. Defaults to false.
 void
-StatusWindow::draw(const std::string &text, bool do_center)
+StatusWindow::draw(const std::wstring &text, bool do_center)
 {
     uint width, height;
     PFont *font = _theme->getStatusData()->getFont(); // convenience
@@ -150,5 +150,6 @@ StatusWindow::center(void)
     Geometry head;
     PScreen::instance()->getHeadInfo(PScreen::instance()->getCurrHead(), head);
 
-    move((head.width - _gm.width) / 2, (head.height - _gm.height) / 2);
+    move(head.x + (head.width - _gm.width) / 2,
+         head.y + (head.height - _gm.height) / 2);
 }
