@@ -1,14 +1,16 @@
 //
 // Workspaces.cc for pekwm
-// Copyright (C) 2002-2004 Claes Nasten <pekdo at pekdon dot net>
+// Copyright © 2002-2008 Claes Nasten <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
-#include "Workspaces.hh"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
 
+#include "Workspaces.hh"
 #include "PScreen.hh"
 #include "Atoms.hh"
 #include "ParseUtil.hh"
@@ -17,8 +19,9 @@
 #include "PDecor.hh"
 #include "Frame.hh"
 #include "Client.hh" // For isSkip()
-#include "Viewport.hh"
 
+#include <iostream>
+#include <sstream>
 #ifdef HAVE_LIMITS
 #include <limits>
 using std::numeric_limits;
@@ -32,25 +35,20 @@ using std::list;
 using std::vector;
 using std::string;
 using std::find;
-#ifdef DEBUG
-#include <iostream>
+using std::wostringstream;
+using std::wstring;
 using std::cerr;
 using std::endl;
-#endif // DEBUG
 
 // Workspaces::Workspace
 
-Workspaces::Workspace::Workspace(const std::string &name, uint number,
-                                 const std::list<PWinObj*> &wo_list) :
-        _name(name), _number(number),
-        _viewport(NULL), _wo_list(wo_list), _last_focused(NULL)
+Workspaces::Workspace::Workspace(const std::wstring &name, uint number, const std::list<PWinObj*> &wo_list)
+  : _name(name), _number(number), _wo_list(wo_list), _last_focused(NULL)
 {
-    _viewport = new Viewport(number, _wo_list);
 }
 
 Workspaces::Workspace::~Workspace(void)
 {
-    delete _viewport;
 }
 
 // Workspaces
@@ -59,11 +57,11 @@ Workspaces *Workspaces::_instance = NULL;
 
 
 //! @brief Workpsaces constructor
-Workspaces::Workspaces(uint number) :
-        _active(0), _previous(0)
+Workspaces::Workspaces(uint number, uint per_row)
+  : _active(0), _previous(0), _per_row(per_row)
 {
 #ifdef DEBUG
-    if (_instance != NULL) {
+    if (_instance) {
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Workspaces(" << this << ")::Workspaces(" << number << ")"
              << endl << " *** _instance allready set: " << _instance << endl;
@@ -82,7 +80,7 @@ Workspaces::Workspaces(uint number) :
 
     // create new workspaces
     for (uint i = 0; i < number; ++i) {
-        _workspace_list.push_back(new Workspace("", i, _wo_list));
+      _workspace_list.push_back(new Workspace(getWorkspaceName(i), i, _wo_list));
     }
 }
 
@@ -133,7 +131,7 @@ Workspaces::setSize(uint number)
 
     } else { // We need more workspaces, lets create some
         for (uint i = before; i < number; ++i) {
-            _workspace_list.push_back(new Workspace("", i, _wo_list));
+          _workspace_list.push_back(new Workspace(getWorkspaceName(i), i, _wo_list));
         }
     }
 
@@ -196,32 +194,61 @@ Workspaces::gotoWorkspace(uint direction, bool warp)
     // Using a bool flag to detect changes due to special workspaces such
     // as PREV
     bool switched = true;
+    uint per_row = Config::instance()->getWorkspacesPerRow();
 
+    uint cur_row = getRow(), row_min = getRowMin(), row_max = getRowMax();
     switch (direction) {
     case WORKSPACE_LEFT:
     case WORKSPACE_PREV:
         dir = 1;
 
-        if (_active > 0) {
-            workspace = _active - 1;
+        if (_active > row_min) {
+          workspace = _active - 1;
         } else if (direction == WORKSPACE_PREV) {
-            workspace = _workspace_list.size() - 1;
+          workspace = row_max;
         } else {
-            switched = false;
+          switched = false;
         }
         break;
     case WORKSPACE_NEXT:
     case WORKSPACE_RIGHT:
-        dir = -1;
+        dir = 2;
 
-        if ((_active + 1) < _workspace_list.size()) {
+        if (_active < row_max) {
             workspace = _active + 1;
         } else if (direction == WORKSPACE_NEXT) {
-            workspace = 0;
+            workspace = row_min;
         } else {
             switched = false;
         }
         break;
+    case WORKSPACE_PREV_V:
+    case WORKSPACE_UP:
+      dir = -1;
+
+      if (_active >= per_row) {
+        workspace = _active - per_row;
+      } else if (direction == WORKSPACE_PREV_V) {
+        // Bottom left
+        workspace = _workspace_list.size() - per_row;
+        // Add column
+        workspace += _active - cur_row * per_row;
+      } else {
+        switched = false;
+      }
+      break;
+    case WORKSPACE_NEXT_V:
+    case WORKSPACE_DOWN:
+      dir = -2;
+
+      if ((_active + per_row) < _workspace_list.size()) {
+        workspace = _active + per_row;
+      } else if (direction == WORKSPACE_NEXT_V) {
+        workspace = _active - cur_row * per_row;
+      } else {
+        switched = false;
+      }
+      break;
     case WORKSPACE_LAST:
         workspace = _previous;
         if (_active == workspace) {
@@ -255,22 +282,27 @@ Workspaces::warpToWorkspace(uint num, int dir)
         return false;
     }
 
-    int x, y, warp;
+    int x, y;
     PScreen::instance()->getMousePosition(x, y);
 
-    warp = Config::instance()->getScreenEdgeSize();
-    warp = (warp > 0) ? (warp * 2) : 2;
-
     if (dir != 0) {
-        if (dir > 0) {
-            x = PScreen::instance()->getWidth() - warp;
-        } else {
-            x = warp;
-        }
+      switch(dir) {
+      case 1:
+        x = PScreen::instance()->getWidth() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_RIGHT) * 2, 2);
+        break;
+      case 2:
+        x = std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_RIGHT) * 2, 2);
+        break;
+      case -1:
+        y = PScreen::instance()->getHeight() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) * 2, 2);;
+        break;
+      case -2:
+        y = std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_TOP) * 2, 2);
+        break;
+      }
 
         // warp pointer
-        XWarpPointer(PScreen::instance()->getDpy(), None,
-                     PScreen::instance()->getRoot(),
+        XWarpPointer(PScreen::instance()->getDpy(), None, PScreen::instance()->getRoot(),
                      0, 0, 0, 0, x, y);
     }
 
@@ -289,7 +321,7 @@ Workspaces::insert(PWinObj* wo, bool raise)
     bool inserted = false;
 
     list<PWinObj*>::iterator it(_wo_list.begin());
-    for (; it != _wo_list.end() && !inserted; ++it) {
+    for (; it != _wo_list.end() && ! inserted; ++it) {
         if (raise
                 ? (wo->getLayer() < (*it)->getLayer())
                 : (wo->getLayer() <= (*it)->getLayer())) {
@@ -301,7 +333,7 @@ Workspaces::insert(PWinObj* wo, bool raise)
         }
     }
 
-    if (!inserted) {
+    if (! inserted) {
         _wo_list.push_back(wo);
         // we can't do raise here as it'll end up in an recursive loop
         // if we use Workspaces::raise to restack the PWinObj.
@@ -329,7 +361,7 @@ Workspaces::hideAll(uint workspace)
 {
     list<PWinObj*>::iterator it(_wo_list.begin());
     for (; it != _wo_list.end(); ++it) {
-        if (((*it)->isSticky() == false) && ((*it)->isHidden() == false) &&
+        if (! ((*it)->isSticky()) && ! ((*it)->isHidden()) &&
                 ((*it)->getWorkspace() == workspace)) {
             (*it)->unmapWindow();
         }
@@ -348,7 +380,7 @@ Workspaces::unhideAll(uint workspace, bool focus)
 
     list<PWinObj*>::iterator it(_wo_list.begin());
     for (; it != _wo_list.end(); ++it) {
-        if (!(*it)->isMapped() && !(*it)->isIconified() && !(*it)->isHidden()
+        if (! (*it)->isMapped() && ! (*it)->isIconified() && ! (*it)->isHidden()
                 && ((*it)->getWorkspace() == workspace)) {
             (*it)->mapWindow(); // don't restack ontop windows
         }
@@ -358,7 +390,7 @@ Workspaces::unhideAll(uint workspace, bool focus)
     // Frame if any and give it focus.
     if (focus) {
         PWinObj *wo = _workspace_list[workspace]->getLastFocused();
-        if (!wo || !PWinObj::windowObjectExists(wo)) {
+        if (! wo || ! PWinObj::windowObjectExists(wo)) {
             wo = getTopWO(PWinObj::WO_FRAME);
         }
 
@@ -381,13 +413,11 @@ Workspaces::unhideAll(uint workspace, bool focus)
         }
 
         // If focusing fails, focus the root window.
-        if (!PWinObj::getFocusedPWinObj()
-                || !PWinObj::getFocusedPWinObj()->isMapped()) {
+        if (! PWinObj::getFocusedPWinObj()
+                || ! PWinObj::getFocusedPWinObj()->isMapped()) {
             PWinObj::getRootPWinObj()->giveInputFocus();
         }
     }
-
-    _workspace_list[_active]->getViewport()->hintsUpdate();
 }
 
 //! @brief Raises a PWinObj and restacks windows.
@@ -480,8 +510,10 @@ Workspaces::getLastFocused(uint workspace)
 void
 Workspaces::setLastFocused(uint workspace, PWinObj* wo)
 {
-    if (workspace >= _workspace_list.size())
+    if (workspace >= _workspace_list.size()) {
         return;
+    }
+    
     _workspace_list[workspace]->setLastFocused(wo);
 }
 
@@ -491,13 +523,26 @@ Workspaces::setLastFocused(uint workspace, PWinObj* wo)
 void
 Workspaces::stackWinUnderWin(Window over, Window under)
 {
-    if (over == under)
+    if (over == under) {
         return;
-
+    }
+    
     Window windows[2] = { over, under };
     XRestackWindows(PScreen::instance()->getDpy(), windows, 2);
 }
 
+//! @brief Create name for workspace num
+wstring
+Workspaces::getWorkspaceName(uint num)
+{
+  wostringstream buf;
+  buf << num + 1;
+  buf << L": ";
+  buf << Config::instance()->getWorkspaceName(num);
+
+  return buf.str();
+}
+
 // MISC METHODS
 
 //! @brief Returns the first focusable PWinObj with the highest stacking
@@ -534,7 +579,7 @@ Workspaces::updateClientStackingList(bool client, bool stacking)
     for (; it != _wo_list.end(); ++it) {
         if ((*it)->getType() == PWinObj::WO_FRAME) {
             cl = static_cast<Client*>(static_cast<Frame*>((*it))->getActiveChild());
-            if (cl && !cl->skipTaskbar()) {
+            if (cl && ! cl->isSkip(SKIP_TASKBAR)) {
                 windows[pos++] = cl->getWindow();
             }
         }
@@ -624,11 +669,11 @@ Workspaces::placeSmart(PWinObj* wo)
 
     if (Config::instance()->getPlacementRow()) { // row placement
         test_y = start_y;
-        while (!placed && (Config::instance()->getPlacementTtB()
+        while (! placed && (Config::instance()->getPlacementTtB()
                            ? ((test_y + wo_height) <= (head.y + head.height))
                            : (test_y >= head.y))) {
             test_x = start_x;
-            while (!placed && (Config::instance()->getPlacementLtR()
+            while (! placed && (Config::instance()->getPlacementLtR()
                                ? ((test_x + wo_width) <= (head.x + head.width))
                                : (test_x >= head.x))) {
                 // see if we can place the window here
@@ -645,11 +690,11 @@ Workspaces::placeSmart(PWinObj* wo)
         }
     } else { // column placement
         test_x = start_x;
-        while (!placed && (Config::instance()->getPlacementLtR()
+        while (! placed && (Config::instance()->getPlacementLtR()
                            ? ((test_x + wo_width) <= (head.x + head.width))
                            : (test_x >= head.x))) {
             test_y = start_y;
-            while (!placed && (Config::instance()->getPlacementTtB()
+            while (! placed && (Config::instance()->getPlacementTtB()
                                ? ((test_y + wo_height) <= (head.y + head.height))
                                : (test_y >= head.y))) {
                 // see if we can place the window here
@@ -751,7 +796,7 @@ Workspaces::placeCenteredOnParent(PWinObj *wo, Window parent)
     }
 
     PWinObj *wo_s = PWinObj::findPWinObj(parent);
-    if (wo_s != NULL) {
+    if (wo_s) {
         wo->move(wo_s->getX() + wo_s->getWidth() / 2 - wo->getWidth() / 2,
                  wo_s->getY() + wo_s->getHeight() / 2 - wo->getHeight() / 2);
         return true;
@@ -785,7 +830,7 @@ Workspaces::placeInsideScreen(Geometry &gm)
 PWinObj*
 Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
 {
-    if (wo == NULL) {
+    if (! wo) {
         return NULL;
     }
 
@@ -794,7 +839,7 @@ Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
     for (; it != _wo_list.end(); ++it) {
         // Skip ourselves, non-mapped and desktop objects. Iconified means
         // skip placement.
-        if (wo == (*it) || !(*it)->isMapped() || (*it)->isIconified()
+        if (wo == (*it) || ! (*it)->isMapped() || (*it)->isIconified()
             || ((*it)->getLayer() == LAYER_DESKTOP)) {
             continue;
         }
@@ -846,7 +891,7 @@ Workspaces::findDirectional(PWinObj *wo, DirectionType dir, uint skip)
 
     list<PWinObj*>::iterator it(_wo_list.begin());
     for (; it != _wo_list.end(); ++it) {
-        if ((wo == (*it)) || ((*it)->isMapped() == false)) {
+        if ((wo == (*it)) || ! ((*it)->isMapped())) {
             continue; // skip ourselves and unmapped wo's
         }
         if (((*it)->getType() != PWinObj::WO_FRAME) ||
